#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_deconvolve import deconvolve
from task_deconvolve import casalog

class deconvolve_pg_:
    __name__ = "deconvolve"

    def __init__(self) :
       self.__bases__ = (deconvolve_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, imagename=None,model=None,psf=None,alg=None,niter=None,gain=None,threshold=None,mask=None,scales=None,sigma=None,targetflux=None,prior=None,):

        """Image based deconvolver

        Several algorithms are available to deconvolve an image with a
        known psf (dirty beam), or a Gaussian beam.  The algorithms
        available are clark and hogbom clean, a multiscale clean and a
        mem clean.  For more deconvolution control, use clean.

    	Keyword arguments:
    	imagename -- Name of input image to be deconvolved
    	model     -- Name of output image containing the clean components
    	psf       -- Name of psf image (dirty beam) to use
                     example: psf='casaxmlf.image' .
     	             If the psf has 3 parameter, then a Gaussian
       		     psf is assumed with the values representing
                     the major , minor and position angle  values
                     e.g  psf=['3arcsec', '2.5arcsec', '10deg']
    	alg       -- algorithm to use: default = 'clark'
                       options: clark, hogbom, multiscale or mem.
    	niter     -- Maximum number of iterations
    	gain      -- CLEAN gain parameter; fraction to remove from peak
    	threshold -- Halt deconvolution if the maximum residual image is
                     below this threshold.
                     default = '0.0Jy'
    	mask      -- mask image (same shape as image and psf) to limit region
       	             where deconvoltion is to occur

    	------parameters useful for multiscale only
    	scales     -- in pixel numbers; the size of component to deconvolve.
                      default value [0,3,10]
                      recommended sizes are 0 (point), 3 (points per clean beam), and
                      10 (about a factor of three lower resolution)
    	------parameters useful for mem only
    	sigma      -- Estimated noise for image
    	targetflux -- Target total flux in image 
    	prior      -- Prior image to guide mem


  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'deconvolve'
        myf['taskname'] = 'deconvolve'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['imagename'] = imagename = myf['imagename']
            myparams['model'] = model = myf['model']
            myparams['psf'] = psf = myf['psf']
            myparams['alg'] = alg = myf['alg']
            myparams['niter'] = niter = myf['niter']
            myparams['gain'] = gain = myf['gain']
            myparams['threshold'] = threshold = myf['threshold']
            myparams['mask'] = mask = myf['mask']
            myparams['scales'] = scales = myf['scales']
            myparams['sigma'] = sigma = myf['sigma']
            myparams['targetflux'] = targetflux = myf['targetflux']
            myparams['prior'] = prior = myf['prior']

        if type(psf)==str: psf=[psf]
        if type(scales)==int: scales=[scales]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['model'] = model
        mytmp['psf'] = psf
        mytmp['alg'] = alg
        mytmp['niter'] = niter
        mytmp['gain'] = gain
        if type(threshold) == str :
           mytmp['threshold'] = casac.quanta().quantity(threshold)
        else :
           mytmp['threshold'] = threshold
        mytmp['mask'] = mask
        mytmp['scales'] = scales
        if type(sigma) == str :
           mytmp['sigma'] = casac.quanta().quantity(sigma)
        else :
           mytmp['sigma'] = sigma
        if type(targetflux) == str :
           mytmp['targetflux'] = casac.quanta().quantity(targetflux)
        else :
           mytmp['targetflux'] = targetflux
        mytmp['prior'] = prior
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'deconvolve.xml')

        casalog.origin('deconvolve')
        if not trec.has_key('deconvolve') or not casac.utils().verify(mytmp, trec['deconvolve']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: deconvolve           #####')
          casalog.post('')
          result = deconvolve(imagename, model, psf, alg, niter, gain, threshold, mask, scales, sigma, targetflux, prior)
          casalog.post('')
          casalog.post('##### End Task: deconvolve           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('deconvolve', 'deconvolve.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'deconvolve'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('deconvolve', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['imagename']  = ''
        a['model']  = ''
        a['psf']  = ['']
        a['alg']  = 'clark'
        a['niter']  = 10
        a['gain']  = 0.1
        a['threshold']  = '0.0mJy'
        a['mask']  = ''

        a['alg'] = {
                    0:{'value':'clark'}, 
                    1:{'value':'hogbom'}, 
                    2:odict([{'value':'multiscale'}, {'scales':[0, 3, 10]}]), 
                    3:odict([{'value':'mem'}, {'sigma':1.0}, {'targetflux':1.0}, {'prior':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='deconvolve', subkey=None):
        desc={'deconvolve': 'Image based deconvolver',
               'imagename': 'Input image to deconvolve',
               'model': 'Output image containing deconvolved point model',
               'psf': 'Point spread function (dirty beam)',
               'alg': 'Algorithm to use (clark, hogbom, multiscale, mem) ',
               'niter': 'number of iteration in deconvolution process',
               'gain': 'CLEAN gain parameter',
               'threshold': 'level below which sources will not be deconvolved',
               'mask': 'image mask to limit region of deconvolution',
               'scales': 'scale sizes (pixels) to deconvolve',
               'sigma': 'mem parameter: Expected noise in image',
               'targetflux': 'mem parameter: Estimated total flux in image',
               'prior': 'mem parameter: prior image for mem search',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['model']  = ''
        a['psf']  = ['']
        a['alg']  = 'clark'
        a['niter']  = 10
        a['gain']  = 0.1
        a['threshold']  = '0.0mJy'
        a['mask']  = ''
        a['scales']  = [0, 3, 10]
        a['sigma']  = '0.0mJy'
        a['targetflux']  = '1.0Jy'
        a['prior']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
deconvolve_pg = deconvolve_pg_()
