#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_exportuvfits import exportuvfits
from task_exportuvfits import casalog

class exportuvfits_pg_:
    __name__ = "exportuvfits"

    def __init__(self) :
       self.__bases__ = (exportuvfits_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,fitsfile=None,datacolumn=None,field=None,spw=None,antenna=None,timerange=None,avgchan=None,writesyscal=None,multisource=None,combinespw=None,writestation=None,padwithflags=None,):

        """Convert a CASA visibility data set to a UVFITS file:

        This task writes a UVFITS file, a general format data set used
        to transfer data between different software systems.  It is
        written in floating point format.  Different programs have different
        restrictions on what forms of UVFITS files they will use, especially
        whether they will accept multiple sources and/or spectral windows in
        the same file.  See the spw, multisource, and combinespw descriptions
        below.

        Keyword arguments:
        vis -- Name of input visibility file
                default: none;  example: vis='ngc5921.ms'
        fitsfile -- Name of output UV FITS file
                default: none;  example='3C273XC1.fits'
        datacolumn -- Visibility file data column
                default: => 'corrected'; example: datacolumn='model'
                Options: 'data' (raw),'corrected','model','weight'
        field -- Select field using field id(s) or field name(s).
                  [run listobs to obtain the list id's or names]
               default: ''=all fields
               If field string is a non-negative integer, it is assumed a field index
               otherwise, it is assumed a field name
               field='0~2'; field ids 0,1,2
               field='0,4,5~7'; field ids 0,4,5,6,7
               field='3C286,3C295'; field named 3C286 adn 3C295
               field = '3,4C*'; field id 3, all names starting with 4C
        spw -- Select spectral window/channels 
                 type 'help par.selection' for more examples.
               spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
               spw='<2';  spectral windows less than 2 (i.e. 0,1)
               spw='0:5~61'; spw 0, channels 5 to 61, INCLUSIVE
               spw='*:5~61'; all spw with channels 5 to 62
               spw='0,10,3:3~45'; spw 0,10 all channels, spw 3, channels 3 to 45.
               spw='0~2:2~6'; spw 0,1,2 with channels 2 through 6 in each.
               spw='0:0~10;15~60'; spectral window 0 with channels 0-10,15-60
                         NOTE ';' to separate channel selections
               spw='0:0~10^2,1:20~30^5'; spw 0, channels 0,2,4,6,8,10,
                     spw 1, channels 20,25,30
        antenna -- Select data based on antenna/baseline
               default: '' (all)
               If antenna string is a non-negative integer, it is assumed
	          an antenna index
                 otherwise, it is assumed as an antenna name
               antenna='5&6'; baseline between antenna index 5 and index 6.
               antenna='VA05&VA06'; baseline between VLA antenna 5 and 6.
               antenna='5&6;7&8'; baseline 5-6 and 7-8
               antenna='5'; all baselines with antenna index 5
               antenna='05'; all baselines with antenna name '05', vla antenna
               antenna='5,6,10'; all baselines with antennas 5, 6 and 10
        timerange  -- Select data based on time range:
               default = '' (all); examples,
               timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
               Note: if YYYY/MM/DD is missing dat defaults to first day in data set
               timerange='09:14:0~09:54:0' picks 40 min on first day
               timerange='25:00:00~27:30:00' picks 1 hr to 3 hr 30min on next day
               timerange='09:44:00' data within one integration of time
               timerange='>10:24:00' data after this time
        avgchan -- Channel averaging width (value > 1 indicates averaging)
                default =>1; example: avgchan=3
                output data will average channels in groups of three.
        multisource -- Write in multi-source format
                default: => True;
                   false if one source is selected
                True works with AIPS, but not difmap.
        combinespw -- If True, export the spectral windows as IFs.
                      Otherwise multiple windows will use multiple FREQIDs.
                default: =>  True;
                   all spectral windows must have same shape.
                True is recommended for AIPS, and mandatory for difmap.
        padwithflags -- If True, and combinespw is True, fill in missing
                        data as needed to fit the IF structure.  This is
                        appropriate if the MS had a few frequency-dependent
                        flags applied, and was then time-averaged by split, or
                        when exporting for use by difmap.  If the spectral
                        windows were observed at different times,
                        padwithflags=True will add a large number of flags,
                        making the output file significantly longer.  It does
                        not yet support spectral windows with different widths.
        writestation -- Write station name instead of antenna name
                default: True;
        writesyscal -- Write GC and TY tables
                default: => False; Not yet available
        async --  Run asynchronously
                default = False;


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'exportuvfits'
        myf['taskname'] = 'exportuvfits'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['fitsfile'] = fitsfile = myf['fitsfile']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['avgchan'] = avgchan = myf['avgchan']
            myparams['writesyscal'] = writesyscal = myf['writesyscal']
            myparams['multisource'] = multisource = myf['multisource']
            myparams['combinespw'] = combinespw = myf['combinespw']
            myparams['writestation'] = writestation = myf['writestation']
            myparams['padwithflags'] = padwithflags = myf['padwithflags']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['fitsfile'] = fitsfile
        mytmp['datacolumn'] = datacolumn
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['antenna'] = antenna
        mytmp['timerange'] = timerange
        mytmp['avgchan'] = avgchan
        mytmp['writesyscal'] = writesyscal
        mytmp['multisource'] = multisource
        mytmp['combinespw'] = combinespw
        mytmp['writestation'] = writestation
        mytmp['padwithflags'] = padwithflags
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'exportuvfits.xml')

        casalog.origin('exportuvfits')
        if not trec.has_key('exportuvfits') or not casac.utils().verify(mytmp, trec['exportuvfits']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: exportuvfits           #####')
          casalog.post('')
          result = exportuvfits(vis, fitsfile, datacolumn, field, spw, antenna, timerange, avgchan, writesyscal, multisource, combinespw, writestation, padwithflags)
          casalog.post('')
          casalog.post('##### End Task: exportuvfits           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('exportuvfits', 'exportuvfits.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'exportuvfits'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('exportuvfits', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['fitsfile']  = ''
        a['datacolumn']  = 'corrected'
        a['field']  = ''
        a['spw']  = ''
        a['antenna']  = ''
        a['timerange']  = ''
        a['avgchan']  = 1
        a['writesyscal']  = False
        a['multisource']  = True
        a['combinespw']  = True
        a['writestation']  = True

        a['combinespw'] = {
                    0:odict([{'value':True}, {'padwithflags':True}]), 
                    1:{'value':False}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='exportuvfits', subkey=None):
        desc={'exportuvfits': 'Convert a CASA visibility data set to a UVFITS file:',
               'vis': 'Name of input visibility file',
               'fitsfile': 'Name of output UV FITS file',
               'datacolumn': 'Visibility file data column',
               'field': 'Select field using field id(s) or field name(s)',
               'spw': 'Select spectral window/channels',
               'antenna': 'Select data based on antenna/baseline',
               'timerange': 'Select data based on time range',
               'avgchan': 'Channel averaging width (value > 1 indicates averaging)',
               'writesyscal': 'Write GC and TY tables, (Not yet available)',
               'multisource': 'Write in multi-source format',
               'combinespw': 'Export the spectral windows as IFs',
               'writestation': 'Write station name instead of antenna name',
               'padwithflags': 'Fill in missing data with flags to fit IFs',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['fitsfile']  = ''
        a['datacolumn']  = 'corrected'
        a['field']  = ''
        a['spw']  = ''
        a['antenna']  = ''
        a['timerange']  = ''
        a['avgchan']  = 1
        a['writesyscal']  = False
        a['multisource']  = True
        a['combinespw']  = True
        a['writestation']  = True
        a['padwithflags']  = False

        if a.has_key(paramname) :
	      return a[paramname]
exportuvfits_pg = exportuvfits_pg_()
