#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_fixplanets import fixplanets
from task_fixplanets import casalog

class fixplanets_pg_:
    __name__ = "fixplanets"

    def __init__(self) :
       self.__bases__ = (fixplanets_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,field=None,fixuvw=None,direction=None,refant=None,reftime=None,):

        """Changes FIELD and SOURCE table entries based on user-provided direction or POINTING table, optionally fixes the UVW coordinates

    Examples:

    fixplanets('uid___A002_X1c6e54_X223.ms', 'Titan', True)
          will look up the pointing direction from antenna 0 for field 'Titan' in 
          the POINTING table based on the first timestamp in the main table rows for 
          this field, enter this direction in the FIELD and SOURCE tables, and then 
          recalculate the UVW coordinates for this field.

    fixplanets('uid___A002_X1c6e54_X223.ms', 'Titan', True, 'Titan_55438-56292dUTC.tab')
          will attach the ephemeris table 'Titan_55438-56292dUTC.tab' to field 'Titan'
          and then recalculate the UVW coordinates for this field.

    fixplanets('uid___A002_X1c6e54_X223.ms', 'Titan', False, 'J2000 12h30m15 -02d12m00')
          will set the directions for field 'Titan' in the FIELD and SOURCE table to the
          given direction and not recalculate the UVW coordinates.
          (This can be useful for several purposes, among them preparing a concatenation
          of datasets. Only fields with the same direction will be recognised as identical.
          fixplanets can then be run again after the concatenation using parameters as in
          the first example above.)


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'fixplanets'
        myf['taskname'] = 'fixplanets'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['field'] = field = myf['field']
            myparams['fixuvw'] = fixuvw = myf['fixuvw']
            myparams['direction'] = direction = myf['direction']
            myparams['refant'] = refant = myf['refant']
            myparams['reftime'] = reftime = myf['reftime']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['field'] = field
        mytmp['fixuvw'] = fixuvw
        mytmp['direction'] = direction
        mytmp['refant'] = refant
        mytmp['reftime'] = reftime
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'fixplanets.xml')

        casalog.origin('fixplanets')
        if not trec.has_key('fixplanets') or not casac.utils().verify(mytmp, trec['fixplanets']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: fixplanets           #####')
          casalog.post('')
          result = fixplanets(vis, field, fixuvw, direction, refant, reftime)
          casalog.post('')
          casalog.post('##### End Task: fixplanets           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('fixplanets', 'fixplanets.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'fixplanets'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('fixplanets', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['field']  = ""
        a['fixuvw']  = False
        a['direction']  = ''
        a['refant']  = 0
        a['reftime']  = 'first'


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='fixplanets', subkey=None):
        desc={'fixplanets': 'Changes FIELD and SOURCE table entries based on user-provided direction or POINTING table, optionally fixes the UVW coordinates',
               'vis': 'Name of the input visibility set.',
               'field': 'Fields to operate on.  Blank = all.',
               'fixuvw': 'recalc uvw?',
               'direction': 'if set, don\'t use pointing table but set direction to this value',
               'refant': 'if using pointing table information, use it from this antenna',
               'reftime': 'if using pointing table information, use it from this timestamp (\'first\', \'median\', or YYYY/MM/DD/hh:mm:ss)',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['field']  = ""
        a['fixuvw']  = False
        a['direction']  = ''
        a['refant']  = 0
        a['reftime']  = 'first'

        if a.has_key(paramname) :
	      return a[paramname]
fixplanets_pg = fixplanets_pg_()
