#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_fixvis import fixvis
from task_fixvis import casalog

class fixvis_pg_:
    __name__ = "fixvis"

    def __init__(self) :
       self.__bases__ = (fixvis_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,outputvis=None,field=None,refcode=None,reuse=None,phasecenter=None,distances=None,datacolumn=None,):

        """Recalculates (u, v, w) and/or changes Phase Center 

If the phase center is changed, the corresponding modifications are applied to the 
visibility columns given by the parameter "datacolumn" which is by default set 
to "all" (DATA, CORRECTED, and MODEL).

    Input Parameters
    vis        -- Name of the input visibility set
    
    outputvis  -- Name of the output visibility set, default: same as vis

    field      -- field selection string

    refcode    -- Reference frame to convert to, default: the refcode of PHASE_DIR in the 
                  FIELD table
                  example: 'B1950'
    
    reuse      -- base recalculation on existing UVW coordinates? default=True
                  ignored if parameter 'phasecenter' is set

    phasecenter  --  if set to a valid direction: change the phase center for the given 
                     field to this value
	       example: 'J2000 9h25m00s -05d12m00s'
               If given without the equinox, e.g. '0h01m00s +00d12m00s', the parameter
               is interpreted as a pair of offsets in RA and DEC to the present 
               phasecenter. 
               NOTE: The RA offset can be given in units of time or angle. If given 
               as a time (i.e. as a single number with a time unit as in, e.g., 12s), 
               it is applied as is. If given as an angle (e.g., 0.01deg or also (!) 00h01m03s), 
               it is corrected for the cos(DEC) before it is applied.

    distances -- (experimental) List of the distances (as quanta) of the fields selected by field
                 to be used for refocussing.
                 If empty, the distances of all fields are assumed to be infinity.
                 If not a list but just a single value is given, this is applied to
                 all fields.
                 default: []
                 examples: ['2E6km', '3E6km']   '15au'

    datacolumn -- when applying a phase center shift, modify visibilities only in 
                  this/these column(s)
                  default: 'all' (DATA, CORRECTED, and MODEL)
                  example: 'DATA,CORRECTED' (will not modify MODEL)
	       
    Examples:

    fixvis('NGC3256.ms','NGC3256-fixed.ms')
          will recalculate the UVW coordinates for all fields based on the existing
          phase center information in the FIELD table.

    fixvis('Moon.ms','Moon-fixed.ms','Moon', '', 'J2000 9h25m00s 05d12m00s')
          will set the phase center for field 'Moon' to the given direction and recalculate
          the UVW coordinates.


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'fixvis'
        myf['taskname'] = 'fixvis'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['outputvis'] = outputvis = myf['outputvis']
            myparams['field'] = field = myf['field']
            myparams['refcode'] = refcode = myf['refcode']
            myparams['reuse'] = reuse = myf['reuse']
            myparams['phasecenter'] = phasecenter = myf['phasecenter']
            myparams['distances'] = distances = myf['distances']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['outputvis'] = outputvis
        mytmp['field'] = field
        mytmp['refcode'] = refcode
        mytmp['reuse'] = reuse
        mytmp['phasecenter'] = phasecenter
        mytmp['distances'] = distances
        mytmp['datacolumn'] = datacolumn
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'fixvis.xml')

        casalog.origin('fixvis')
        if not trec.has_key('fixvis') or not casac.utils().verify(mytmp, trec['fixvis']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: fixvis           #####')
          casalog.post('')
          result = fixvis(vis, outputvis, field, refcode, reuse, phasecenter, distances, datacolumn)
          casalog.post('')
          casalog.post('##### End Task: fixvis           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('fixvis', 'fixvis.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'fixvis'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('fixvis', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['outputvis']  = ''
        a['field']  = ""
        a['refcode']  = ''
        a['reuse']  = True
        a['phasecenter']  = ''
        a['distances']  = ""
        a['datacolumn']  = 'all'


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='fixvis', subkey=None):
        desc={'fixvis': 'Recalculates (u, v, w) and/or changes Phase Center ',
               'vis': 'Name of the input visibility set.',
               'outputvis': 'Name of the output visibility set.  (Can be the same as vis.)',
               'field': 'Fields to operate on.  '' = all.',
               'refcode': 'reference frame to convert UVW coordinates to',
               'reuse': 'base UVW calculation on the old values?',
               'phasecenter': 'use this direction as phase center',
               'distances': '(experimental) List of the distances (as quanta) of the fields selected by field.',
               'datacolumn': 'when applying a phase center shift, modify visibilities only in this/these column(s)',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['outputvis']  = ''
        a['field']  = ""
        a['refcode']  = ''
        a['reuse']  = True
        a['phasecenter']  = ''
        a['distances']  = ""
        a['datacolumn']  = 'all'

        if a.has_key(paramname) :
	      return a[paramname]
fixvis_pg = fixvis_pg_()
