#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_hanningsmooth import hanningsmooth
from task_hanningsmooth import casalog

class hanningsmooth_pg_:
    __name__ = "hanningsmooth"

    def __init__(self) :
       self.__bases__ = (hanningsmooth_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,datacolumn=None,outputvis=None,):

        """Hanning smooth frequency channel data to remove Gibbs ringing

    This function Hanning smoothes the frequency channels with
    a weighted running average. The weights are 0.5 for the central 
    channel and 0.25 for each of the two adjacent channels. The first 
    and last channels are flagged.
    Inclusion of a flagged value in an average causes that data value
    to be flagged. 
    If an 'outputvis' filename is given, the task will copy the input file to the 
    output file name first, including all columns that are present in the input MS. 
    After that step it will smooth the column(s) as requested in the 'datacolumn' parameter. 
    Alternatively, if no 'outputvis' is specified, hanningsmooth will work directly on the 
    input visibility file. 
    If the 'CORRECTED' data column is requested for an MS that does not contain this column, 
    it will be filled from the 'DATA' column and then smoothed.
    
    WARNING: by default, all visibility columns will be smoothed. This will 
             modify the DATA column of the output MS in order to make sure that
             later gaincal will work on the smoothed data, e.g. as part of self-cal.

    Keyword arguments:
    vis -- Name of input visibility file (MS)
           default: none; example: vis='ngc5921.ms'
    datacolumn -- the name of the MS column to be Hanning smoothed
                  default='all'; example: datacolumn='corrected'
                  options: 'corrected', 'data', 'all'
    outputvis -- name of the output visibility file (MS)
                 default=none (write to the input MS); example: outputvis='ngc5921_src.ms'

    hanningsmooth(vis='ngc4852.ms', datacolumn='data', outputvis='ngc4852-hs.ms')		 


 
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'hanningsmooth'
        myf['taskname'] = 'hanningsmooth'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['outputvis'] = outputvis = myf['outputvis']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['datacolumn'] = datacolumn
        mytmp['outputvis'] = outputvis
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'hanningsmooth.xml')

        casalog.origin('hanningsmooth')
        if not trec.has_key('hanningsmooth') or not casac.utils().verify(mytmp, trec['hanningsmooth']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: hanningsmooth           #####')
          casalog.post('')
          result = hanningsmooth(vis, datacolumn, outputvis)
          casalog.post('')
          casalog.post('##### End Task: hanningsmooth           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('hanningsmooth', 'hanningsmooth.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'hanningsmooth'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('hanningsmooth', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['datacolumn']  = 'all'
        a['outputvis']  = ''


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='hanningsmooth', subkey=None):
        desc={'hanningsmooth': 'Hanning smooth frequency channel data to remove Gibbs ringing',
               'vis': 'Name of input visibility file (MS)',
               'datacolumn': 'the name of the MS column into which to write the smoothed data',
               'outputvis': 'name of the output visibility file (MS)',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['datacolumn']  = 'all'
        a['outputvis']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
hanningsmooth_pg = hanningsmooth_pg_()
