#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_imcollapse import imcollapse
from task_imcollapse import casalog

class imcollapse_pg_:
    __name__ = "imcollapse"

    def __init__(self) :
       self.__bases__ = (imcollapse_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, imagename=None,function=None,axes=None,outfile=None,box=None,region=None,chans=None,stokes=None,mask=None,overwrite=None,stretch=None,):

        """Collapse image along one axis, aggregating pixel values along that axis.
PARAMETER SUMMARY
imagename        Name of the input (CASA, FITS, MIRIAD) image
function         Function used to compute aggregation of pixel values along the collapsed
                 axis. Supported functions are flux, max, mean, median, min, rms, stdev,
                 sum, variance. Minimum match is supported for the function parameter (eg,
                 function="r" will compute the rms of the pixel values).
axis             Zero-based axis number(s) or minimal match strings to compress.
outfile          Name of output CASA image. Must be specified.
overwrite        Controls if an already existing file by the
                 same name can be overwritten. If true, the user is not prompted, the file
                 if it exists is automatically overwritten.
box              Direction plane box specification, "blcx, blcy, trcx, trcy". Only one box
                 may be specified. If not specified, region is used if specified. If region
                 is also not specified, entire directional plane unioned with any chans and
                 stokes specification determines the region.
region           Region specification. See help par.region. Default is to not use a region.
chans            Optional contiguous frequency channel number specification. Not used if
                 region is specified. Default is all channels. See "help par.chans" for examples.
stokes           Contiguous stokes planes specification. Not used if region is specified.
                 Default is all stokes.
mask             Mask to use. See help par.mask. Default is none.
stretch          Stretch the input mask if necessary and possible. Only used if a mask is specified.
                 See help par.stretch.

This task collapses an image along a specified axis or set of axes of N pixels to a single pixel on each
specified axis. Both float valued and complex valued images are supported. It computes the specified
aggregate function for pixel values along the specified axes
and places those values in the single remaining plane of those axes in the output image. It returns
an image analysis tool containing the newly created collapsed image if wantreturn=True. Choices of
aggregate functions are: flux (see below for constraints), max, mean, median, min, rms, stdev, sum, variance. Minimal unique match is
supported for the function parameter (eg, function="r" will compute the rms of the pixel values, "med"
will compute the median).

If one specifies function='flux', the following constraints must be true:

1. The image must have a direction coordinate,
2. The image must have at least one beam,
3. The specified axes must be exactly the direction coordinate axes,
4. Only one of the non-directional axes may be non-degenerate,
5. The iamge brightness unit must be conformant with x*yJy/beam, where x is an optional unit (such as km/s for moments images)
   and y is an optional SI prefix.
        
Axes can be specified as a single integer or array of integers indicating the zero-based axes along
which to collapse the image. Axes may also be specified as a single or array of strings which minimally
and uniquely match (ignoring case) world axes names in the image (eg "dec" or ["ri, "d"] for
collapsing along the declination axis or along the right ascension and declination axes, respectively).
         
The reference pixel of the collapsed axis is set to 0 and its reference value is set to the mean
of the the first and last values of that axis in the specified region of the input image.

# myimage.im is a 512x512x128x4 (ra,dec,freq,stokes) image
imagename = "myimage.im"
# collapse a subimage of it along its spectral axis avoiding the 8 edge
# channels at each end of the band, computing the mean value of the pixels
# resulting image is 256x256x1x4 in size.
outfile="collapse_spec_mean.im"
function="mean"
axis=2
box="127,127,383,383"
chans="8~119"
imcollapse(imagename=imagename, outfile=outfile, function=function, axes=axis, box=box, chans=chans)


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'imcollapse'
        myf['taskname'] = 'imcollapse'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['imagename'] = imagename = myf['imagename']
            myparams['function'] = function = myf['function']
            myparams['axes'] = axes = myf['axes']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['box'] = box = myf['box']
            myparams['region'] = region = myf['region']
            myparams['chans'] = chans = myf['chans']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['mask'] = mask = myf['mask']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['stretch'] = stretch = myf['stretch']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['function'] = function
        mytmp['axes'] = axes
        mytmp['outfile'] = outfile
        mytmp['box'] = box
        mytmp['region'] = region
        mytmp['chans'] = chans
        mytmp['stokes'] = stokes
        mytmp['mask'] = mask
        mytmp['overwrite'] = overwrite
        mytmp['stretch'] = stretch
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'imcollapse.xml')

        casalog.origin('imcollapse')
        if not trec.has_key('imcollapse') or not casac.utils().verify(mytmp, trec['imcollapse']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: imcollapse           #####')
          casalog.post('')
          result = imcollapse(imagename, function, axes, outfile, box, region, chans, stokes, mask, overwrite, stretch)
          casalog.post('')
          casalog.post('##### End Task: imcollapse           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('imcollapse', 'imcollapse.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'imcollapse'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('imcollapse', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['imagename']  = ''
        a['function']  = ''
        a['axes']  = [0]
        a['outfile']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''
        a['mask']  = ''
        a['stretch']  = False

        a['outfile'] = {
                    0:odict([{'notvalue':''}, {'overwrite':False}])}
        a['box'] = {
                    0:odict([{'value':''}, {'region':""}])}
        a['mask'] = {
                    0:odict([{'notvalue':''}, {'stretch':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='imcollapse', subkey=None):
        desc={'imcollapse': 'Collapse image along one axis, aggregating pixel values along that axis.',
               'imagename': 'Name of the input image',
               'function': 'Aggregate function to apply. This can be set one of flux, max, mean, median, min, rms, stdev, sum, variance. Must be specified.',
               'axes': 'Zero-based axis number(s) or minimal match strings to collapse.',
               'outfile': 'Name of output CASA image. Must be specified.',
               'box': 'Optional direction plane box ("blcx, blcy, trcx trcy").',
               'region': 'Region specification. See help par.region. Default is to not use a region.',
               'chans': 'Optional zero-based contiguous frequency channel specification. See "help par.chans" for examples.',
               'stokes': 'Optional contiguous stokes planes specification.',
               'mask': 'Mask to use. See help par.mask. Default is none.',
               'overwrite': 'Overwrite exisitng ouput file if it exists?',
               'stretch': 'Stretch the mask if necessary and possible? See help par.stretch ',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['function']  = ''
        a['axes']  = [0]
        a['outfile']  = ''
        a['box']  = ''
        a['region']  = ''
        a['chans']  = ''
        a['stokes']  = ''
        a['mask']  = ''
        a['overwrite']  = False
        a['stretch']  = False

        if a.has_key(paramname) :
	      return a[paramname]
imcollapse_pg = imcollapse_pg_()
