#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_imcontsub import imcontsub
from task_imcontsub import casalog

class imcontsub_pg_:
    __name__ = "imcontsub"

    def __init__(self) :
       self.__bases__ = (imcontsub_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, imagename=None,linefile=None,contfile=None,fitorder=None,region=None,box=None,chans=None,stokes=None,):

        """Estimates and subtracts continuum emission from an image cube

  For each (x, y) column in imagename (or a subset selected by region and/or
  box), this estimates the continuum by fitting a polynomial to one or more
  subsets of the channels.  The continuum estimate is saved in contfile, and
  subtracted from imagename (or its subset) to make a spectral line estimate,
  which is saved in linefile.

  
  

  Keyword arguments:
  imagename -- Input image cube
    Default: none; Example: imagename='ngc5921_task.im'
  linefile -- Name of output spectral line cube
    Default: none; Example: outline='ngc5921_line.im'
  contfile -- Name of output continuum cube
    Default: none; Example: contfile='ngc5921_cont.im'
  fitorder -- Polynomial order for the continuum estimation.
    Default: 0; Example fitorder=2
  region -- ImageRegion file to select a portion of the cube for output.
    Default: '' (= everything)
    Example: region='myimage.im.rgn'
             region='region1'
    Use the viewer on the image cube to define the continuum
    regions.  This is more general than box but still experimental.
    N.B. This is NOT for selecting a subset of the input for
         fitting the continuum!  Only pixels that are in region
         will be considered, however.

  box --  A [blcx, blcy, trcx, trcy] region on the directional plane for
          selecting a subset for output.
          ONLY pixel values are acceptable at this time.
          Default: none (whole 2-D plane); Example: box='10,10,50,50'
          N.B. This is NOT for selecting a subset of the input for
               fitting the continuum!  Only pixels that are in region
               will be considered, however.
  chans -- line-free channel numbers to fit the continuum to.
           N.B.: This is currently the _only_ way to specify what is
                 continuum vs. line emission.
           ONLY channel numbers accepted at this time, i.e. there is no
           'spw:' as in the spw parameter of other tasks.  For
           a multi-spw image, the channelization must be the same for all.
           Default: '' (all)
           Example: chans='3~6;>40'
  stokes -- Stokes parameters to image, may or may not be separated
            by commas but best if you use commas.
            Default: none (all); Example: stokes='IQUV';
            Options: 'I','Q','U','V','RR','RL','LR','LL','XX','YX','XY','YY', ... 



        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'imcontsub'
        myf['taskname'] = 'imcontsub'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['imagename'] = imagename = myf['imagename']
            myparams['linefile'] = linefile = myf['linefile']
            myparams['contfile'] = contfile = myf['contfile']
            myparams['fitorder'] = fitorder = myf['fitorder']
            myparams['region'] = region = myf['region']
            myparams['box'] = box = myf['box']
            myparams['chans'] = chans = myf['chans']
            myparams['stokes'] = stokes = myf['stokes']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['linefile'] = linefile
        mytmp['contfile'] = contfile
        mytmp['fitorder'] = fitorder
        mytmp['region'] = region
        mytmp['box'] = box
        mytmp['chans'] = chans
        mytmp['stokes'] = stokes
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'imcontsub.xml')

        casalog.origin('imcontsub')
        if not trec.has_key('imcontsub') or not casac.utils().verify(mytmp, trec['imcontsub']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: imcontsub           #####')
          casalog.post('')
          result = imcontsub(imagename, linefile, contfile, fitorder, region, box, chans, stokes)
          casalog.post('')
          casalog.post('##### End Task: imcontsub           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('imcontsub', 'imcontsub.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'imcontsub'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('imcontsub', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['imagename']  = ''
        a['linefile']  = ''
        a['contfile']  = ''
        a['fitorder']  = 0
        a['region']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='imcontsub', subkey=None):
        desc={'imcontsub': 'Estimates and subtracts continuum emission from an image cube',
               'imagename': 'Name of the input spectral line image',
               'linefile': 'Output continuum-subtracted image file name',
               'contfile': 'Output continuum image file name',
               'fitorder': 'Polynomial order for the continuum estimation',
               'region': 'Image region used for output selection',
               'box': '[blcx, blcy, trcx, trcy] for output selection',
               'chans': 'Channel range(s) for continuum fitting',
               'stokes': 'Stokes params to image (I,IV,IQU,IQUV)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['linefile']  = ''
        a['contfile']  = ''
        a['fitorder']  = 0
        a['region']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
imcontsub_pg = imcontsub_pg_()
