#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_importasdm import importasdm
from task_importasdm import casalog

class importasdm_pg_:
    __name__ = "importasdm"

    def __init__(self) :
       self.__bases__ = (importasdm_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, asdm=None,vis=None,singledish=None,antenna=None,corr_mode=None,srt=None,time_sampling=None,ocorr_mode=None,compression=None,lazy=None,asis=None,wvr_corrected_data=None,scans=None,ignore_time=None,process_syspower=None,process_caldevice=None,process_pointing=None,process_flags=None,tbuff=None,applyflags=None,savecmds=None,outfile=None,flagbackup=None,verbose=None,overwrite=None,showversion=None,useversion=None,bdfflags=None,with_pointing_correction=None,remove_ref_undef=None,):

        """Convert an ALMA Science Data Model observation into a CASA visibility file (MS) or single-dish data format (Scantable)		          
Keyword arguments:
asdm -- Name of input ASDM file (directory)
        default: none; example: asdm='ExecBlock3'

vis       -- Root ms or scantable name, note a prefix (.ms or .asap) is NOT appended to this name
             default: none

singledish   -- Set True to write data as single-dish format (Scantable)
                default: False

            >>> singledish expandable parameter
                 antenna -- antenna name or id.
 
corr_mode -- correlation mode to be considered on input. Could
	     be one or more of the following, ao, co, ac, or all
	     default: 'all'

srt       -- spectral resolution type. Could be one or more of
	     the following, fr, ca, bw, or all
	     default: 'all'

time_sampling -- specifies the time sampling, INTEGRATION and/or
                 SUBINTEGRAION. could be one or more of the following
                 i, si, or all.
		 default: 'all'

ocorr_mode    -- output data for correlation mode AUTO_ONLY 
                 (ao) or CROSS_ONLY (co) or CROSS_AND_AUTO (ca)
		 default: 'ca'
		 
compression  -- produces comrpressed columns in the resulting measurement set.
                default: False

lazy         -- Make the MS DATA column read the ASDM Binary data directly
                (faster import, smaller MS)
                default: False

asis         --  creates verbatim copies of the ASDM tables in 
                 the output measurement set. The value given to
		 this option must be a list of table names separated
		 by space characters; the wildcard character '*' is 
                 allowed in table names.
                default: none

wvr_corrected_data -- specifies wich values are considered in the 
                      ASDM binary data to fill the DATA column in 
                      the MAIN table of the MS. Expected values for 
                      this option are 'no' for the uncorrected data 
                      (this is the default), 'yes' for the corrected
                      data and 'both' for corrected and uncorrected 
                      data. In the latter case, two measurement sets
                      are created, one containing the uncorrected 
                      data and the other one, whose name is suffixed
                      by '-wvr-corrected', containing the corrected 
                      data.
                  default: 'no'

scans --	  processes only the scans specified in the option's value. This value is a semicolon 
                  separated list of scan specifications. A scan specification consists in an exec bock index 
                  followed by the character ':' followed by a comma separated list of scan indexes or scan 
                  index ranges. A scan index is relative to the exec block it belongs to. Scan indexes are 
                  1-based while exec blocks's are 0-based. '0:1' or '2:2~6' or '0:1,1:2~6,8;2:,3:24~30' '1,2' 
                  are valid values for the option. '3:' alone will be interpreted as 'all the scans of the 
                  exec block#3'. An scan index or a scan index range not preceded by an exec block index will
                  be interpreted as 'all the scans with such indexes in all the exec blocks'.  By default 
                  all the scans are considered.
                    default: none (all scans)

ignore_time -- All the rows of the tables Feed, History, Pointing, Source, SysCal, CalDevice, SysPower,
               and Weather are processed independently of the time range of the selected exec block / scan.
                    default: False

process_syspower -- The SysPower table is processed if and only if this parameter is set to True.
                    default: True

process_caldevice -- The CalDevice table is processed if and only if this parameter is set to True.
                    default: True

process_pointing -- The Pointing table is processed if and only if this parameter is set to True.
                    default: True

process_flags    -- Create online flags based on the Flag.xml, Antenna.xml and SpectralWindow.xml files
                    and copy them to the FLAG_CMD sub-table of the MS. The flags will NOT be applied unless 
                    the parameter applyflags is set to True. Optionally, the flags can also be saved to
                    an external ASCII file if savecmds is set to True.
                    default: True
                    
            >>> process_flags expandable parameter
                 tbuff   -- Time padding buffer (in seconds)
                    default: 0.0

                 NOTE: this time is in seconds. You should currently
                 set the value of tbuff to be 1.5x the correlator
                 integration time if greater than 1 second.  For
                 example, if the SDM has integrations of 3 seconds,
                 set tbuff=4.5.  Likewise, set tbuff=15.0 for 10-sec
                 integrations.

                 applyflags -- Apply the online flags to the MS.
                   default: False


                savecmds -- Save the flag commands to an ASCII file given by the parameter outfile. 
                  default: False
                    
                outfile -- Filename or list of filenames where to save the online flag commands.
                  default: ' ' --> by default it will save on a filename composed from the MS name(s).
                    Example: vis='uid_A02.ms', the outfile will be 'uid_A02_cmd.txt'. 
                             vis='uid_A02-wvr-corrected.ms', the outfile will be 'uid_A02-wvr-corrected_cmd.txt'.  
                    

flagbackup   -- Backup original flags in >ms<.flagversions 
                default: True
  
verbose     -- produce log output as asdm2MS is being run
                default: False
      
overwrite   -- overwrite an existing MS or MS(s), if the option wvr_corrected_data='both'
                default: False (do not overwrite)
      
                NOTE: the overwrite parameter affects all the output of the task. If any of the following
                      exist, it will not overwrite them. MS(s), .flagversions, online flag files. When set to
                      True, it will overwrite the MS, .flagversions and online flag file.      

showversion -- report the version of the asdm2MS being used.
                    default: False

useversion -- Selects the version of asdm2MS to be used (\'v3\' (default, should work for all data))
              default: v3

bdfflags -- Set the MS FLAG column according to the ASDM _binary_ flags
            default: false

with_pointing_correction -- add (ASDM::Pointing::encoder - ASDM::Pointing::pointingDirection) to the value to be written 
                            in MS::Pointing::direction 

remove_ref_undef -- if set to True then apply fixspwbackport on the resulting MSes.
	            default: False
           
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'importasdm'
        myf['taskname'] = 'importasdm'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['asdm'] = asdm = myf['asdm']
            myparams['vis'] = vis = myf['vis']
            myparams['singledish'] = singledish = myf['singledish']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['corr_mode'] = corr_mode = myf['corr_mode']
            myparams['srt'] = srt = myf['srt']
            myparams['time_sampling'] = time_sampling = myf['time_sampling']
            myparams['ocorr_mode'] = ocorr_mode = myf['ocorr_mode']
            myparams['compression'] = compression = myf['compression']
            myparams['lazy'] = lazy = myf['lazy']
            myparams['asis'] = asis = myf['asis']
            myparams['wvr_corrected_data'] = wvr_corrected_data = myf['wvr_corrected_data']
            myparams['scans'] = scans = myf['scans']
            myparams['ignore_time'] = ignore_time = myf['ignore_time']
            myparams['process_syspower'] = process_syspower = myf['process_syspower']
            myparams['process_caldevice'] = process_caldevice = myf['process_caldevice']
            myparams['process_pointing'] = process_pointing = myf['process_pointing']
            myparams['process_flags'] = process_flags = myf['process_flags']
            myparams['tbuff'] = tbuff = myf['tbuff']
            myparams['applyflags'] = applyflags = myf['applyflags']
            myparams['savecmds'] = savecmds = myf['savecmds']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['flagbackup'] = flagbackup = myf['flagbackup']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['showversion'] = showversion = myf['showversion']
            myparams['useversion'] = useversion = myf['useversion']
            myparams['bdfflags'] = bdfflags = myf['bdfflags']
            myparams['with_pointing_correction'] = with_pointing_correction = myf['with_pointing_correction']
            myparams['remove_ref_undef'] = remove_ref_undef = myf['remove_ref_undef']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['asdm'] = asdm
        mytmp['vis'] = vis
        mytmp['singledish'] = singledish
        mytmp['antenna'] = antenna
        mytmp['corr_mode'] = corr_mode
        mytmp['srt'] = srt
        mytmp['time_sampling'] = time_sampling
        mytmp['ocorr_mode'] = ocorr_mode
        mytmp['compression'] = compression
        mytmp['lazy'] = lazy
        mytmp['asis'] = asis
        mytmp['wvr_corrected_data'] = wvr_corrected_data
        mytmp['scans'] = scans
        mytmp['ignore_time'] = ignore_time
        mytmp['process_syspower'] = process_syspower
        mytmp['process_caldevice'] = process_caldevice
        mytmp['process_pointing'] = process_pointing
        mytmp['process_flags'] = process_flags
        mytmp['tbuff'] = tbuff
        mytmp['applyflags'] = applyflags
        mytmp['savecmds'] = savecmds
        mytmp['outfile'] = outfile
        mytmp['flagbackup'] = flagbackup
        mytmp['verbose'] = verbose
        mytmp['overwrite'] = overwrite
        mytmp['showversion'] = showversion
        mytmp['useversion'] = useversion
        mytmp['bdfflags'] = bdfflags
        mytmp['with_pointing_correction'] = with_pointing_correction
        mytmp['remove_ref_undef'] = remove_ref_undef
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'importasdm.xml')

        casalog.origin('importasdm')
        if not trec.has_key('importasdm') or not casac.utils().verify(mytmp, trec['importasdm']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: importasdm           #####')
          casalog.post('')
          result = importasdm(asdm, vis, singledish, antenna, corr_mode, srt, time_sampling, ocorr_mode, compression, lazy, asis, wvr_corrected_data, scans, ignore_time, process_syspower, process_caldevice, process_pointing, process_flags, tbuff, applyflags, savecmds, outfile, flagbackup, verbose, overwrite, showversion, useversion, bdfflags, with_pointing_correction, remove_ref_undef)
          casalog.post('')
          casalog.post('##### End Task: importasdm           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('importasdm', 'importasdm.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'importasdm'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('importasdm', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['asdm']  = ''
        a['vis']  = ''
        a['singledish']  = False
        a['corr_mode']  = 'all'
        a['srt']  = 'all'
        a['time_sampling']  = 'all'
        a['ocorr_mode']  = 'ca'
        a['compression']  = False
        a['lazy']  = False
        a['asis']  = ''
        a['wvr_corrected_data']  = 'no'
        a['scans']  = ''
        a['ignore_time']  = False
        a['process_syspower']  = True
        a['process_caldevice']  = True
        a['process_pointing']  = True
        a['process_flags']  = True
        a['flagbackup']  = True
        a['verbose']  = False
        a['overwrite']  = False
        a['showversion']  = False
        a['useversion']  = 'v3'
        a['bdfflags']  = False
        a['with_pointing_correction']  = False
        a['remove_ref_undef']  = False

        a['singledish'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'antenna':0}])}
        a['process_flags'] = {
                    0:odict([{'value':True}, {'tbuff':0.0}, {'applyflags':False}, {'savecmds':False}, {'outfile':''}]), 
                    1:{'value':False}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='importasdm', subkey=None):
        desc={'importasdm': 'Convert an ALMA Science Data Model observation into a CASA visibility file (MS) or single-dish data format (Scantable)',
               'asdm': 'Name of input asdm directory (on disk)',
               'vis': 'Root name of the ms to be created. Note the .ms is NOT added ',
               'singledish': 'Set true to output single-dish data format ',
               'antenna': 'antenna name or id',
               'corr_mode': 'specifies the correlation mode to be considered on input. A quoted string containing a sequence of ao, co, ac,or all separated by whitespaces is expected',
               'srt': 'specifies the spectral resolution type to be considered on input. A quoted string containing a sequence of fr, ca, bw, or all separated by whitespaces is expected',
               'time_sampling': 'specifies the time sampling (INTEGRATION and/or SUBINTEGRATION)  to be considered on input. A quoted string containing a sequence of i, si, or all separated by whitespaces is expected',
               'ocorr_mode': 'output data for correlation mode AUTO_ONLY (ao) or CROSS_ONLY (co) or CROSS_AND_AUTO (ca)',
               'compression': 'Flag for turning on data compression',
               'lazy': 'Make the MS DATA column read the ASDM Binary data directly (faster import, smaller MS)',
               'asis': 'Creates verbatim copies of the ASDMtables in the ouput measurement set.  Value given must be a string of table names separated by spaces; A * wildcard is allowed.',
               'wvr_corrected_data': 'Specifies which values are considerd in the SDM binary data to fill the DATA column in the MAIN table of the MS. Expected values for this option are: no, for uncorrected data (default), yes, for the corrected data, and both, for for corrected and uncorrected data. Note if both is selected two measurement sets are created, one with uncorrected data and the other with corrected data.  ',
               'scans': 'processes only the specified scans. This value is a semicolon separated list of scan specifications. A scan specification consists in an exec bock index followed by the : character;  followed by a comma separated list of scan indexes or scan index ranges. A scan index is relative to the exec block it belongs to. Scan indexes are 1-based while exec blocks are 0-based. "0:1" or "2:2~6" or "0:1,1:2~6,8;2:,3:24~30" "1,2" are valid values for the option. "3:" alone will be interpreted as, all the scans of the exec block#3.  An scan index or a scan index range not preceded by an exec block index will be interpreted as, all the scans with such indexes in all the exec blocks.  By default all the scans are considered. ',
               'ignore_time': 'All the rows of the tables Feed, History, Pointing, Source, SysCal, CalDevice, SysPower, and Weather are processed independently of the time range of the selected exec block / scan.',
               'process_syspower': ' The SysPower table is processed if and only if this parameter is set to true.',
               'process_caldevice': 'The CalDevice table is processed if and only if this parameter is set to true.',
               'process_pointing': 'The Pointing table is processed if and only if this parameter is set to true. If set to False, the POINTING table is empty in the resulting MS',
               'process_flags': 'Create online flags in the FLAG_CMD sub-table.',
               'tbuff': ' Time padding buffer (seconds)',
               'applyflags': 'Apply the flags to the MS.',
               'savecmds': 'Save flag commands to an ASCII file',
               'outfile': 'Name of ASCII file to save flag commands',
               'flagbackup': 'Back up flag column before applying flags.',
               'verbose': 'Output lots of information while the filler is working',
               'overwrite': 'Over write an existing MS(s)',
               'showversion': 'Report the version of asdm2MS being used',
               'useversion': 'Version of asdm2MS to be used (\'v3\' (default, should work for all data))',
               'bdfflags': 'Set the MS FLAG column according to the ASDM _binary_ flags',
               'with_pointing_correction': ' add (ASDM::Pointing::encoder - ASDM::Pointing::pointingDirection) to the value to be written in MS::Pointing::direction',
               'remove_ref_undef': 'if set to True then apply fixspwbackport on the resulting MS(es).',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['asdm']  = ''
        a['vis']  = ''
        a['singledish']  = False
        a['antenna']  = 0
        a['corr_mode']  = 'all'
        a['srt']  = 'all'
        a['time_sampling']  = 'all'
        a['ocorr_mode']  = 'ca'
        a['compression']  = False
        a['lazy']  = False
        a['asis']  = ''
        a['wvr_corrected_data']  = 'no'
        a['scans']  = ''
        a['ignore_time']  = False
        a['process_syspower']  = True
        a['process_caldevice']  = True
        a['process_pointing']  = True
        a['process_flags']  = True
        a['tbuff']  = 0.0
        a['applyflags']  = False
        a['savecmds']  = True
        a['outfile']  = ''
        a['flagbackup']  = True
        a['verbose']  = False
        a['overwrite']  = False
        a['showversion']  = False
        a['useversion']  = 'v3'
        a['bdfflags']  = False
        a['with_pointing_correction']  = False
        a['remove_ref_undef']  = False

        if a.has_key(paramname) :
	      return a[paramname]
importasdm_pg = importasdm_pg_()
