#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_initweights import initweights
from task_initweights import casalog

class initweights_pg_:
    __name__ = "initweights"

    def __init__(self) :
       self.__bases__ = (initweights_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,wtmode=None,dowtsp=None,):

        """Initializes weight information in the MS

      This task provides for initialization of the weight information
      in the MS.  For ALMA and EVLA data, it should not generally be
      necessary to use this task, as the weight information should 
      have been initialized properly at fill time (v4.2.2 and later).

      Several initialization modes are supported via the wtmode parameter.

      If wtmode='nyq' (the default), SIGMA and WEIGHT will be
      initialized according to bandwidth and integration time.  This
      is the theoretically correct mode for raw normalized visibilities.
      (e.g., ALMA).  For the EVLA, this is correct if switched-power
      and bandpass calibration will later be applied.  

      If wtmode='sigma', WEIGHT will be initialized according to the
      existing SIGMA column.  

      If mode='weight', WEIGHT_SPECTRUM will be initialized according
      to the existing WEIGHT column; dowtspec=T must be specified in
      this case.
 
      If wtmode='ones', SIGMA and WEIGHT will be initialized with 1.0,
      globally.  This is a traditional means of initializing weight
      information, and is adequate when the integration time and 
      bandwidth are uniform. It is not recommended for modern
      instruments (ALMA, EVLA), where variety in observational setups
      is common, and properly initialized and calibrated weights
      will be used for imaging sensitivity estimates.

      For the above wtmodes, if dowtspec=T (or if the WEIGHT_SPECTRUM
      column already exists), the WEIGHT_SPECTRUM column will be
      initialized (uniformly in channel), in a manner consistent with
      the disposition of the WEIGHT column.  If the WEIGHT_SPECTRUM 
      column does not exist, dowtsp=T will force its creation.
      Use of the WEIGHT_SPECTRUM column is only meaningful
      for ALMA data which will be calibrated with channelized
      Tsys information, or if the weights will become channelized
      after calibration, e.g., via averaging over time- and 
      channel-dependent flagging.  (A task for channel-dependent
      weight estimation from the data itself is also currently under
      development).

      Two additional modes are available for managing the spectral
      weight info columns; these should be used with extreme care: If
      wtmode='delwtsp', the WEIGHT_SPECTRUM column will be deleted (if
      it exists).  If wtmode='delsigsp', the SIGMA_SPECTRUM column
      will be deleted (if it exists).  Note that creation of
      SIGMA_SPECTRUM is not supported via this method.

      Note that this task does not support any prior selection.  
      Intialization of the weight information must currently be done 
      globally or not at all.  This is to maintain consistency.

 
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'initweights'
        myf['taskname'] = 'initweights'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['wtmode'] = wtmode = myf['wtmode']
            myparams['dowtsp'] = dowtsp = myf['dowtsp']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['wtmode'] = wtmode
        mytmp['dowtsp'] = dowtsp
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'initweights.xml')

        casalog.origin('initweights')
        if not trec.has_key('initweights') or not casac.utils().verify(mytmp, trec['initweights']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: initweights           #####')
          casalog.post('')
          result = initweights(vis, wtmode, dowtsp)
          casalog.post('')
          casalog.post('##### End Task: initweights           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('initweights', 'initweights.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'initweights'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('initweights', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['wtmode']  = 'nyq'
        a['dowtsp']  = False


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='initweights', subkey=None):
        desc={'initweights': 'Initializes weight information in the MS',
               'vis': 'Name of input visibility file (MS)',
               'wtmode': 'Initialization mode',
               'dowtsp': 'Initialize the WEIGHT_SPECTRUM column.',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['wtmode']  = 'nyq'
        a['dowtsp']  = False

        if a.has_key(paramname) :
	      return a[paramname]
initweights_pg = initweights_pg_()
