#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_listobs import listobs
from task_listobs import casalog

class listobs_pg_:
    __name__ = "listobs"

    def __init__(self) :
       self.__bases__ = (listobs_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,selectdata=None,spw=None,field=None,antenna=None,uvrange=None,timerange=None,correlation=None,scan=None,intent=None,feed=None,array=None,observation=None,verbose=None,listfile=None,listunfl=None,cachesize=None,overwrite=None,):

        """List the summary of a data set in the logger or in a file

       List the summary information of a data set in the logger or in a file, based on
       a data selection. Only rows can be selected and printed. No in-row selection is
       possible (channel or correlation). Refer to the task listvis to list visibilites.

       Lists the following properties of a measurement set:
       scan list, field list, spectral window list with
       correlators, antenna locations, ms table information.

       Keyword arguments:
       vis -- Name of input visibility file
               default: none. example: vis='ngc5921.ms'
       
       selectdata -- Select a subset of data for flagging
                    default: False
                    options: True,False
                    The summary listing will only apply to the specified selection.

              antenna -- Select data based on baseline
                    default: '' (all); example: antenna='5&6' baseline 5-6
                    antenna='5&6;7&8' #baseline 5-6 and 7-8
                    antenna='5' # all cross-correlation baselines between antenna 5 and all other available
                                  antennas
                    antenna='5,6' # all baselines with antennas 5 and 6
                    antenna='1&&1' # only the auto-correlation baselines for antenna 1
                    antenna='1&&*' # cross and auto-correlation baselines between antenna 1
                                             and all other available antennas
                    antenna='1~7&&&' # only the auto-correlation baselines for antennas in range 1~7
              spw -- Select data based on spectral window and channels
                    default: '' (all); example: spw='1'
                    spw='<2' #spectral windows less than 2
                    spw='>1' #spectral windows greater than 1
              correlation -- Correlation types
                    default: '' (all);
                    example: correlation='RR LL'
              field -- Select data based on field id(s) or name(s)
                    default: '' (all); example: field='1'
                    field='0~2' # field ids inclusive from 0 to 2
                    field='3C*' # all field names starting with 3C
              uvrange -- Select data within uvrange (default units meters)
                    default: '' (all); example:
                    uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lamgda
                    uvrange='>4klamda';uvranges greater than 4 kilo-lambda
                    uvrange='0~1000km'; uvrange in kilometers
              timerange  -- Select data based on time range:
                    default = '' (all); example,
                    timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                    Note: YYYY/MM/DD can be dropped as needed:
                    timerange='09:14:0~09:54:0' # this time range
                    timerange='09:44:00' # data within one integration of time
                    timerange='>10:24:00' # data after this time
                    timerange='09:44:00+00:13:00' #data 13 minutes after time
              scan -- Select data based on scan number
                    default: '' (all); example: scan='>3'
              intent -- Select data based on observation intent
                    default: '' (all); example: intent='*CAL*,*BAND*'
              feed -- Selection based on the feed - NOT IMPLEMENTED YET
              array -- Selection based on the antenna array
              observation -- Selection based on the observation ID
                    default: '' (all); example: observation='1' or observation=1


       verbose -- level of detail
             verbose=True: (default); scan and antenna lists
             verbose=False: less information
             
       listfile -- name of disk file to write output.
               default: None. Example: listfile='list.txt'
               
       listunfl -- List unflagged row counts? If true, it can have significant negative performance impact.
 
       cachesize -- maximum size of the memory cache in megabytes in which data structures can be
                    stored. For very large datasets this can be increased for possibly better performance.
                    THIS IS ONLY EXPERIEMENTAL FOR NOW, AND INCREASING THE VALUE OF THIS PARAMETER DOES NOT GUARANTEE INCREASED
                    SPEED. DEPENDING ON ITS (LACK OF) USEFULNESS, IT MAY BE REMOVED IN THE FUTURE.


      The 'Int (s)' column is the average of the MS's INTERVAL column
      for each scan, so in a time-averaged MS 'Int' = 9.83s more likely
      means 5 10s integrations and 1 9s integration (timebin) than 6
      9.83s integrations. 
      
    DESCRIPTION OF ALGORITHM TO CALCULATE THE NUMBER OF UNFLAGGED ROWS
    The number of unflagged rows are only computed if listunfl=True. Computing these quantity
    can have a negative performance impact, especially for large datasets.
    The number of unflagged rows (the nUnflRows columns in the scans and fields portions of the listing) is
    calculated by summing the fractional unflagged bandwidth for each row (and hence why the number of unflagged
    rows, in general, is not an integer). Thus a row which has half of its
    total bandwidth flagged contributes 0.5 rows to the unflagged row count. A row with 20 of 32 channels of
    homogeneous width contributes 20/32 = 0.625 rows to the unflagged row count. A row with a value of False
    in the FLAG_ROW column is not counted in the number of unflagged rows.
 
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'listobs'
        myf['taskname'] = 'listobs'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['selectdata'] = selectdata = myf['selectdata']
            myparams['spw'] = spw = myf['spw']
            myparams['field'] = field = myf['field']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['correlation'] = correlation = myf['correlation']
            myparams['scan'] = scan = myf['scan']
            myparams['intent'] = intent = myf['intent']
            myparams['feed'] = feed = myf['feed']
            myparams['array'] = array = myf['array']
            myparams['observation'] = observation = myf['observation']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['listfile'] = listfile = myf['listfile']
            myparams['listunfl'] = listunfl = myf['listunfl']
            myparams['cachesize'] = cachesize = myf['cachesize']
            myparams['overwrite'] = overwrite = myf['overwrite']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['selectdata'] = selectdata
        mytmp['spw'] = spw
        mytmp['field'] = field
        mytmp['antenna'] = antenna
        mytmp['uvrange'] = uvrange
        mytmp['timerange'] = timerange
        mytmp['correlation'] = correlation
        mytmp['scan'] = scan
        mytmp['intent'] = intent
        mytmp['feed'] = feed
        mytmp['array'] = array
        mytmp['observation'] = observation
        mytmp['verbose'] = verbose
        mytmp['listfile'] = listfile
        mytmp['listunfl'] = listunfl
        mytmp['cachesize'] = cachesize
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'listobs.xml')

        casalog.origin('listobs')
        if not trec.has_key('listobs') or not casac.utils().verify(mytmp, trec['listobs']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: listobs           #####')
          casalog.post('')
          result = listobs(vis, selectdata, spw, field, antenna, uvrange, timerange, correlation, scan, intent, feed, array, observation, verbose, listfile, listunfl, cachesize, overwrite)
          casalog.post('')
          casalog.post('##### End Task: listobs           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('listobs', 'listobs.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'listobs'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('listobs', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['selectdata']  = True
        a['verbose']  = True
        a['listfile']  = ''
        a['listunfl']  = False
        a['cachesize']  = 50

        a['selectdata'] = {
                    0:odict([{'value':True}, {'field':''}, {'spw':''}, {'antenna':''}, {'timerange':''}, {'correlation':''}, {'scan':''}, {'intent':''}, {'feed':''}, {'array':''}, {'uvrange':''}, {'observation':''}]), 
                    1:{'value':False}}
        a['listfile'] = {
                    0:odict([{'notvalue':''}, {'overwrite':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='listobs', subkey=None):
        desc={'listobs': 'List the summary of a data set in the logger or in a file',
               'vis': 'Name of input visibility file (MS)',
               'selectdata': 'Data selection parameters',
               'spw': 'spectral-window/frequency/channel',
               'field': 'Field names or field index numbers: \'\'==>all, field=\'0~2,3C286\'',
               'antenna': 'antenna/baselines: \'\'==>all, antenna =\'3,VA04\'',
               'uvrange': 'uv range: \'\'==>all; uvrange =\'0~100klambda\', default units=meters',
               'timerange': 'time range: \'\'==>all,timerange=\'09:14:0~09:54:0\'',
               'correlation': 'Select data based on correlation',
               'scan': 'scan numbers: \'\'==>all',
               'intent': 'Select data based on observation intent: \'\'==>all',
               'feed': 'multi-feed numbers: Not yet implemented',
               'array': '(sub)array numbers: \'\'==>all',
               'observation': 'Select data based on observation ID: \'\'==>all',
               'verbose': '',
               'listfile': 'Name of disk file to write output: \'\'==>to terminal',
               'listunfl': 'List unflagged row counts? If true, it can have significant negative performance impact.',
               'cachesize': 'EXPERIMENTAL. Maximum size in megabytes of cache in which data structures can be held.',
               'overwrite': 'If True, tacitly overwrite listfile if it exists.',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['selectdata']  = True
        a['spw']  = ''
        a['field']  = ''
        a['antenna']  = ''
        a['uvrange']  = ''
        a['timerange']  = ''
        a['correlation']  = ''
        a['scan']  = ''
        a['intent']  = ''
        a['feed']  = ''
        a['array']  = ''
        a['observation']  = ''
        a['verbose']  = True
        a['listfile']  = ''
        a['listunfl']  = False
        a['cachesize']  = 50
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
listobs_pg = listobs_pg_()
