#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_listsdm import listsdm
from task_listsdm import casalog

class listsdm_pg_:
    __name__ = "listsdm"

    def __init__(self) :
       self.__bases__ = (listsdm_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, sdm=None,):

        """Lists observation information present in an SDM directory.

	The listsdm task reads SDM XML tables, processes the
	observation information contained therein, and prints this
	information to the CASA log.  It will also return a dictionary
	keyed on scan number.  The dictionary contains the following
	information:

	'baseband'   list of baseband name(s)
	'chanwidth'  list of channel widths (Hz)
	'end'        observation end time (UTC)
	'field'      field ID
	'intent'     scan intent(s)
	'nchan'      list of number of channels
	'nsubs'      number of subscans
	'reffreq'    list of reference frequencies (Hz)
	'source'     source name
	'spws'       list of spectral windows
	'start'      observation start time (UTC)
	'timerange'  start time - end time range (UTC)
	
	Example:

	myscans = listsdm(sdm='AS1039_sb1382796_2_000.55368.51883247685')

	Prints information about the requested SDM to the CASA logger
	and returns a dictionary with scan information in 'myscans'.

        Keyword argument:

        sdm -- Name of input SDM directory.  
               example: sdm='AG836_sb1377811_1.55345.300883159725'
	       	
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'listsdm'
        myf['taskname'] = 'listsdm'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['sdm'] = sdm = myf['sdm']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['sdm'] = sdm
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'listsdm.xml')

        casalog.origin('listsdm')
        if not trec.has_key('listsdm') or not casac.utils().verify(mytmp, trec['listsdm']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: listsdm           #####')
          casalog.post('')
          result = listsdm(sdm)
          casalog.post('')
          casalog.post('##### End Task: listsdm           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('listsdm', 'listsdm.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'listsdm'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('listsdm', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['sdm']  = ''


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='listsdm', subkey=None):
        desc={'listsdm': 'Lists observation information present in an SDM directory.',
               'sdm': 'Name of input SDM directory',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['sdm']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
listsdm_pg = listsdm_pg_()
