#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_listvis import listvis
from task_listvis import casalog

class listvis_pg_:
    __name__ = "listvis"

    def __init__(self) :
       self.__bases__ = (listvis_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,options=None,datacolumn=None,field=None,spw=None,selectdata=None,antenna=None,timerange=None,correlation=None,scan=None,feed=None,array=None,observation=None,uvrange=None,average=None,showflags=None,pagerows=None,listfile=None,):

        """List measurement set visibilities.

This task lists measurement set visibility data under a number of
input selection conditions.  The measurement set data columns that 
can be listed are: the raw data, float_data, corrected data, model data, 
and residual (corrected - model) data.

The output table format is dynamic.  Field, Spectral Window, and
Channel columns are not displayed if the column contents are uniform.
For example, if "spw = '1'" is specified, the spw column will not be
displayed.  When a column is not displayed, a message is sent to the
logger and terminal indicating that the column values are uniform and 
listing the uniform value.

Table column descriptions:

COLUMN NAME       DESCRIPTION
-----------       -----------
Date/Time         Time stamp of data sample (YYMMDD/HH:MM:SS UT)
Intrf             Interferometer baseline (antenna names)
UVDist            uv-distance (units of wavelength)
Fld               Field ID (if more than 1)
SpW               Spectral Window ID (if more than 1)
Chn               Channel number (if more than 1)
(Correlated       Correlated polarizations (eg: RR, LL, XY)
  polarization)     Sub-columns are: Amp, Phs, Wt, F
Amp               Visibility amplitude
Phs               Visibility phase (deg)
Wt                Weight of visibility measurement
F                 Flag: 'F' = flagged datum; ' ' = unflagged
UVW               UVW coordinates (meters)


Input Parameters:
vis         Name of input visibility file
            default: none; example: vis='ngc5921.ms'

options     List options: default = 'ap'
            Not yet implemented for suboptions

datacolumn  Visibility file data column:
            default = 'data':  options are
            data, float_data, corrected, model, residual (corrected-model)

field       Select data based on field id(s) or name(s)
            default: ''==>all; example: field='1'
            field='0~2' field ids inclusive from 0 to 2
            field='3C*' all field names starting with 3C

spw         Select spectral window, channel to list
            default: '0:0' --> spw=0, chan=0
            spw='2:34' spectral window 2, channel 34

selectdata  Toggle the following 7 selection parameters.
            default: False; example: selectdata=True
            If false, the following parameters are reset
            to default values.

      antenna     Select calibration data based on antenna
                  default: ''-->all; examples: 
                  antenna = '5,6'; antenna index 5 and 6 solutions
                  antenna = '05,06'; antenna names '05' and '06 solutions
                 
      timerange   Select time range to list
                  default: ''-->all; examples:
                  timerange='10:37:50.1'; list data for this sampling interval
                  timerange='<10:37:25'; list data before 10:37:25
      
      correlation Select polarization correlations to list
                  default: ''-->all; examples: 
                  correlation='RR LL'; list RR and LL correlations
                  correlation='XX XY'; list XX and XY correlations
      
      scan        Select scans to list
                  default: ''-->all; examples:
                  scan='2'; list scan 2
                  scan='>2'; list scan numbers greater than 2
      
      feed        (not yet implemented)
      
      array       (not yet implemented)

      observation Select by observation ID(s).
                  default: ''-->all;
		  example: observation='0' (select obsID 0)
      
      uvrange     Select baseline lengths to list.
                  default: ''--> all; examples:
                  uvrange='<5kl'; less than 5 kilo-wavelengths
                  Caution: Input units default to meters.
                  Listed units are always wavelengths.

average     (not yet implemented)

showflags   (not yet implemented)

pagerows    rows per page of listing
            default: 50; 0 --> do not paginate

listfile    write output to disk; will not overwrite
            default: '' --> write to screen
            listfile = 'solutions.txt'

async       Run asynchronously
            default = False; do not run asychronously

   
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'listvis'
        myf['taskname'] = 'listvis'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['options'] = options = myf['options']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['selectdata'] = selectdata = myf['selectdata']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['correlation'] = correlation = myf['correlation']
            myparams['scan'] = scan = myf['scan']
            myparams['feed'] = feed = myf['feed']
            myparams['array'] = array = myf['array']
            myparams['observation'] = observation = myf['observation']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['average'] = average = myf['average']
            myparams['showflags'] = showflags = myf['showflags']
            myparams['pagerows'] = pagerows = myf['pagerows']
            myparams['listfile'] = listfile = myf['listfile']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['options'] = options
        mytmp['datacolumn'] = datacolumn
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['selectdata'] = selectdata
        mytmp['antenna'] = antenna
        mytmp['timerange'] = timerange
        mytmp['correlation'] = correlation
        mytmp['scan'] = scan
        mytmp['feed'] = feed
        mytmp['array'] = array
        mytmp['observation'] = observation
        mytmp['uvrange'] = uvrange
        mytmp['average'] = average
        mytmp['showflags'] = showflags
        mytmp['pagerows'] = pagerows
        mytmp['listfile'] = listfile
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'listvis.xml')

        casalog.origin('listvis')
        if not trec.has_key('listvis') or not casac.utils().verify(mytmp, trec['listvis']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: listvis           #####')
          casalog.post('')
          result = listvis(vis, options, datacolumn, field, spw, selectdata, antenna, timerange, correlation, scan, feed, array, observation, uvrange, average, showflags, pagerows, listfile)
          casalog.post('')
          casalog.post('##### End Task: listvis           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('listvis', 'listvis.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'listvis'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('listvis', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['options']  = 'ap'
        a['datacolumn']  = 'data'
        a['field']  = ''
        a['spw']  = '*'
        a['selectdata']  = False
        a['observation']  = ''
        a['average']  = ''
        a['showflags']  = False
        a['pagerows']  = 50
        a['listfile']  = ''

        a['selectdata'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'antenna':''}, {'timerange':''}, {'correlation':''}, {'scan':''}, {'feed':''}, {'array':''}, {'observation':''}, {'uvrange':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='listvis', subkey=None):
        desc={'listvis': 'List measurement set visibilities.',
               'vis': 'Name of input visibility file',
               'options': 'List options: ap only ',
               'datacolumn': 'Column to list: data, float_data, corrected, model, residual',
               'field': 'Field names or index to be listed: \'\'==>all',
               'spw': 'Spectral window:channels: \'\*\'==>all, spw=\'1:5~57\'',
               'selectdata': 'Other data selection parameters',
               'antenna': 'Antenna/baselines: \'\'==>all, antenna = \'3\'',
               'timerange': 'Time range: \'\'==>all',
               'correlation': 'Correlations: \'\'==>all, correlation = \'RR RL\'',
               'scan': 'Scan numbers',
               'feed': 'Multi-feed numbers (Not yet implemented)',
               'array': 'Array numbers (Not yet implemented)',
               'observation': 'Select by observation ID(s)',
               'uvrange': 'uv range: \'\'==>all; not yet implemented ',
               'average': 'Averaging mode: ''==>none (Not yet implemented)',
               'showflags': 'Show flagged data (Not yet implemented)',
               'pagerows': 'Rows per page',
               'listfile': 'Output file',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['options']  = 'ap'
        a['datacolumn']  = 'data'
        a['field']  = ''
        a['spw']  = '*'
        a['selectdata']  = False
        a['antenna']  = ''
        a['timerange']  = ''
        a['correlation']  = ''
        a['scan']  = ''
        a['feed']  = ''
        a['array']  = ''
        a['observation']  = ''
        a['uvrange']  = ''
        a['average']  = ''
        a['showflags']  = False
        a['pagerows']  = 50
        a['listfile']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
listvis_pg = listvis_pg_()
