#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_msmoments import msmoments
from task_msmoments import casalog

class msmoments_pg_:
    __name__ = "msmoments"

    def __init__(self) :
       self.__bases__ = (msmoments_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,moments=None,antenna=None,field=None,spw=None,includemask=None,excludemask=None,outfile=None,overwrite=None,):

        """Compute moments from an MS
        The spectral moment distributions at each row in input MS are
        determined. Input MS must have FLOAT_DATA column, i.e. 
        autocorrelation data.  
        See the cookbook and User Reference Manual for
        mathematical details.

        The main control of the calculation is given by parameter
        moments:
	
        moments=-1  - mean value of the spectrum
        moments=0   - integrated value of the spectrum
        moments=1   - intensity weighted coordinate;traditionally used to get 
                      'velocity fields'
        moments=2   - intensity weighted dispersion of the coordinate; traditionally
                      used to get "velocity dispersion"
        moments=3   - median of I
        moments=4   - median coordinate
        moments=5   - standard deviation about the mean of the spectrum
        moments=6   - root mean square of the spectrum
        moments=7   - absolute mean deviation of the spectrum
        moments=8   - maximum value of the spectrum
        moments=9   - coordinate of the maximum value of the spectrum
        moments=10  - minimum value of the spectrum
        moments=11  - coordinate of the minimum value of the spectrum

        Note that includemask and excludemask cannot set simultaneously. 

        Keyword arguments:
        infile -- Name of input MS data
                default: none; example: infile="OrionS_rawACSmod"
        moments -- List of moments you would like to compute
                default: 0 (integrated spectrum);example: moments=[0,1]
                see list above
        antenna -- antenna name or id that the user wants to compute moments
                default: '' (all antennae)
        field -- field name or id that the user wants to compute moments
                default: '' (all fields)
        spw -- spectral window id that the user wants to compute moments
                default: '' (all spectral windows)

        includemask -- List of masks to include
                default: [-1] (include all channels); example=[2,100]
        excludemask -- List of masks to exclude
                default: [-1] (don't exclude channels); example=[100,200]
        outfile -- Output MS file name (or root for multiple moments)
                default: '' (input+auto-determined suffix);example: outfile='source_moment'
        overwrite -- Overwrite existing output files
                default: false

        Example for finding the 1-momment, intensity-weighted
        coordinate, often used for finding velocity fields.
        msmoments( infile='mydata', moment=1, outfile='velocityfields' )


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'msmoments'
        myf['taskname'] = 'msmoments'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['moments'] = moments = myf['moments']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['includemask'] = includemask = myf['includemask']
            myparams['excludemask'] = excludemask = myf['excludemask']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(moments)==int: moments=[moments]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['moments'] = moments
        mytmp['antenna'] = antenna
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['includemask'] = includemask
        mytmp['excludemask'] = excludemask
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'msmoments.xml')

        casalog.origin('msmoments')
        if not trec.has_key('msmoments') or not casac.utils().verify(mytmp, trec['msmoments']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: msmoments           #####')
          casalog.post('')
          result = msmoments(infile, moments, antenna, field, spw, includemask, excludemask, outfile, overwrite)
          casalog.post('')
          casalog.post('##### End Task: msmoments           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('msmoments', 'msmoments.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'msmoments'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('msmoments', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['moments']  = [0]
        a['antenna']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['includemask']  = -1
        a['excludemask']  = -1
        a['outfile']  = ''
        a['overwrite']  = False


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='msmoments', subkey=None):
        desc={'msmoments': 'Compute moments from an MS',
               'infile': 'Name of the input MS data',
               'moments': 'List of moments you want to compute',
               'antenna': 'antenna name or id',
               'field': 'field name or id',
               'spw': 'spectral window id',
               'includemask': 'Range of rows to include',
               'excludemask': 'Range of rows to exclude',
               'outfile': 'Output file name (or root for multiple moments) ',
               'overwrite': 'Overwrite existing output files',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['moments']  = [0]
        a['antenna']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['includemask']  = -1
        a['excludemask']  = -1
        a['outfile']  = ''
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
msmoments_pg = msmoments_pg_()
