#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdflag import sdflag
from task_sdflag import casalog

class sdflag_pg_:
    __name__ = "sdflag"

    def __init__(self) :
       self.__bases__ = (sdflag_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,mode=None,unflag=None,field=None,spw=None,timerange=None,scan=None,pol=None,beam=None,restfreq=None,frame=None,doppler=None,clipminmax=None,clipoutside=None,showflagged=None,row=None,rasterrow=None,outfile=None,outform=None,overwrite=None,plotlevel=None,):

        """ASAP SD spectral spectral/row flagging task 
Keyword arguments:
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
mode -- type of flag operation
        options: (str) 'manual', 'clip', 'interactive', 'rowid'
        default: 'manual'

     >>> common data selection parameters for all modes except mode='rowid'
        field -- select data by field IDs and names
                default: '' (use all fields)
                example: field='3C2*' (all names starting with 3C2)
                         field='0,4,5~7' (field IDs 0,4,5,6,7)
                         field='0,3C273' (field ID 0 or field named 3C273)
                this selection is in addition to the other selections to data
        spw -- select data by IF IDs (spectral windows)/channels
                NOTE channel range selection is valid only in mode='manual'
                default: '' (use all IFs and channels)
                example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                         spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                         spw='115GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
                         spw='0:5~61' (IF ID 0; channels 5 to 61)
                         spw='3:10~20;50~60' (select multiple channel ranges within IF ID 3)
                         spw='3:10~20,4:0~30' (select different channel ranges for IF IDs 3 and 4)
                         spw='1~4;6:15~48' (for channels 15 through 48 for IF IDs 1,2,3,4 and 6)
                this selection is in addition to the other selections to data
        timerange -- select data by time range
                default: '' (use all)
                example: timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                         Note: YYYY/MM/DD can be dropped as needed:
                         timerange='09:14:00~09:54:00' # this time range
                         timerange='09:44:00' # data within one integration of time
                         timerange='>10:24:00' # data after this time
                         timerange='09:44:00+00:13:00' #data 13 minutes after time
                this selection is in addition to the other selections to data
        scan -- select data by scan numbers
                default: '' (use all scans)
                example: scan='21~23' (scan IDs 21,22,23)
                this selection is in addition to the other selections to data
        pol -- select data by polarization IDs
                default: '' (use all polarizations)
                example: pol='0,1' (polarization IDs 0,1)
                this selection is in addition to the other selections to data
        beam -- select data by beam IDs
                default: '' (use all beams)
                example: beam='0,1' (beam IDs 0,1)
                this selection is in addition to the other selections to data

     >>> common data parameters for all modes except mode='interactive'
        unflag -- flag or unflag
                default: False (flag selected data)
                options: (bool) True, False

     >>> mode='manual' expandable parameters
        restfreq -- the rest frequency (effective only when spw selection is in
                    velocity unit.)
                    available type includes float, int, string, list of float, 
                    list of int, list of string, and list of dictionary. the 
                    default unit of restfreq in case of float, int, or string 
                    without unit is Hz. string input can be a value only 
                    (treated as Hz) or a value followed by unit for which 'GHz',
                    'MHz','kHz',and 'Hz' are available. 
                    a list can be used to set different rest frequencies for 
                    each IF. the length of list input must be number of IFs. 
                    dictionary input should be a pair of line name and 
                    frequency with keys of 'name' and 'value', respectively. 
                    values in the dictionary input follows the same manner as 
                    as for single float or string input. 
                example: 345.796
                         '1420MHz'
                         [345.8, 347.0, 356.7]
                         ['345.8MHz', '347.0MHz', '356.7MHz']
                         [{'name':'CO','value':345}]
        frame -- frequency reference frame (effective only when spw selection is in
                 velocity or frequency unit.)
                options: 'LSRK', 'TOPO', 'LSRD', 'BARY', 'GALACTO', 'LGROUP', 'CMB'
                default: '' (keep current frame in data)
        doppler -- doppler convention (effective only when spw is in
                   velocity unit)
                options: 'RADIO', 'OPTICAL', 'Z', 'BETA', or 'GAMMA'
                default: '' (keep current doppler setting in data)
     >>> mode='clip' expandable parameters
        clipminmax -- range of data that will NOT be flagged
                default: [] (means no clip operation)
                example: [0.0,1.5]
        clipoutside -- clip OUTSIDE the range ?
                options: (bool)True,False
                default: True
                example: clipoutside=False means flag data WITHIN the range.
     >>> mode='interactive' expandable parameters
        showflagged -- show flagged data on plots
                options: (bool) True, False
                default: False
     >>> mode='rowid' expandable parameters
        row -- select data by row IDs to apply flag/unflag in the input scannable
                Note, this parameter is effective only when one or more row 
                IDs are given explicitly
                default: '' (means no selection)
                example: '200~300,400~500' (rows 200 to 300 and 400 to 500)
outfile -- name of output file
        default: ''
        Note: by default (outfile=''), actual output file name is set as follows: 
              (1) if overwrite=True (default), infile (input) will be overwritten.
              WARNING: If the formats of input and ouput files are different, 
                       this causes complete loss of input file.
              (2) if overwrite=False, outfile will be <infile>_f. 
outform -- output file format
        options: 'ASAP','MS2', 'ASCII','SDFITS'
        default: 'ASAP'
        NOTE the ASAP format is easiest for further sd
        processing; use MS2 for CASA imaging.
        If ASCII, then will append some stuff to
        the outfile name
        WARNING: Be sure outform is same as the input file format when you 
                 overwrite the input file by overwrite=True and outfile='' (default).
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: True
        WARNING: input file is overwritten if overwrite=True and outfile='' (default). 
                 This causes the complete loss of input file if the formats of
                 input and ouput files are different.
plotlevel -- control for plotting of results
        options: (int) 0, 1, 2, and their negative counterparts
        default: 0 (no plotting)
        example: plotlevel=1; plot spectra and flagged channels before and after
                         current operation. asked if you accept the flag for each
                         spw. also, the first spectrum after the operation is plotted.
                 plotlevel=2; additionally list scantable before and after operation.
                 plotlevel<0 as abs(plotlevel), e.g.
                 -1 => hardcopy of final plot (will be named
                 <outfile>_flag.eps)
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdflag'
        myf['taskname'] = 'sdflag'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['mode'] = mode = myf['mode']
            myparams['unflag'] = unflag = myf['unflag']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['scan'] = scan = myf['scan']
            myparams['pol'] = pol = myf['pol']
            myparams['beam'] = beam = myf['beam']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['frame'] = frame = myf['frame']
            myparams['doppler'] = doppler = myf['doppler']
            myparams['clipminmax'] = clipminmax = myf['clipminmax']
            myparams['clipoutside'] = clipoutside = myf['clipoutside']
            myparams['showflagged'] = showflagged = myf['showflagged']
            myparams['row'] = row = myf['row']
            myparams['rasterrow'] = rasterrow = myf['rasterrow']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['plotlevel'] = plotlevel = myf['plotlevel']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['mode'] = mode
        mytmp['unflag'] = unflag
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['timerange'] = timerange
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['beam'] = beam
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['clipminmax'] = clipminmax
        mytmp['clipoutside'] = clipoutside
        mytmp['showflagged'] = showflagged
        mytmp['row'] = row
        mytmp['rasterrow'] = rasterrow
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdflag.xml')

        casalog.origin('sdflag')
        if not trec.has_key('sdflag') or not casac.utils().verify(mytmp, trec['sdflag']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdflag           #####')
          casalog.post('')
          result = sdflag(infile, antenna, mode, unflag, field, spw, timerange, scan, pol, beam, restfreq, frame, doppler, clipminmax, clipoutside, showflagged, row, rasterrow, outfile, outform, overwrite, plotlevel)
          casalog.post('')
          casalog.post('##### End Task: sdflag           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdflag', 'sdflag.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdflag'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdflag', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['mode']  = 'manual'
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = True
        a['plotlevel']  = 0

        a['mode'] = {
                    0:odict([{'value':'manual'}, {'field':''}, {'spw':''}, {'timerange':''}, {'scan':''}, {'pol':''}, {'beam':''}, {'rasterrow':''}, {'unflag':False}, {'restfreq':''}, {'frame':''}, {'doppler':''}]), 
                    1:odict([{'value':'clip'}, {'field':''}, {'spw':''}, {'timerange':''}, {'scan':''}, {'pol':''}, {'beam':''}, {'clipminmax':[]}, {'clipoutside':True}, {'unflag':False}]), 
                    2:odict([{'value':'interactive'}, {'field':''}, {'spw':''}, {'timerange':''}, {'scan':''}, {'pol':''}, {'beam':''}, {'showflagged':False}]), 
                    3:odict([{'value':'rowid'}, {'row':''}, {'unflag':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdflag', subkey=None):
        desc={'sdflag': 'ASAP SD spectral spectral/row flagging task ',
               'infile': 'name of input SD dataset',
               'antenna': 'select an antenna name or ID, e.g, \'PM03\' (only effective for MS input)',
               'mode': 'mode of data selection and flag operation',
               'unflag': 'unflag selected data (False: flag, True: unflag)',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\' = all)',
               'spw': 'select data by IF IDs (spectral windows), e.g., \'3,5,7\' (\'\' = all)',
               'timerange': 'select data by time range, e.g, \'09:14:0~09:54:0\' (\'\' = all) (see examples in help)',
               'scan': 'select data by scan numbers, e.g, \'21~23\' (\'\' = all)',
               'pol': 'select data by polarization IDs, e.g, \'0,1\' (\'\' = all)',
               'beam': 'select data by beam IDs, e.g, \'0,1\' (\'\' = all)',
               'restfreq': 'the rest frequency, \'1.41GHz\' (default unit: Hz). Effective only when spw selection is in velocity unit. (see examples in help) ',
               'frame': 'frequency reference frame (\'\'=current) Effective only when spw selection is in velocity or frequency unit.',
               'doppler': 'doppler convention (\'\'=current). Effective only when spw selection is in velocity unit.',
               'clipminmax': 'range of data that will NOT be flagged',
               'clipoutside': 'clip outside the range, or within it',
               'showflagged': 'show flagged data (in gray) on plots',
               'row': 'select data by row IDs to apply row-based flagging/unflagging (e.g., \'0,3,5\')',
               'rasterrow': 'select data by raster rows (e.g., \'0,3,5\')',
               'outfile': 'name of output file (See a WARNING in help)',
               'outform': 'output file format (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists (See a WARNING in help)',
               'plotlevel': 'control for plotting of results (see examples in help)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['mode']  = 'manual'
        a['unflag']  = False
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['beam']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['clipminmax']  = []
        a['clipoutside']  = True
        a['showflagged']  = False
        a['row']  = ''
        a['rasterrow']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = True
        a['plotlevel']  = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdflag_pg = sdflag_pg_()
