#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdimaging import sdimaging
from task_sdimaging import casalog

class sdimaging_pg_:
    __name__ = "sdimaging"

    def __init__(self) :
       self.__bases__ = (sdimaging_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infiles=None,outfile=None,overwrite=None,field=None,spw=None,antenna=None,scan=None,mode=None,nchan=None,start=None,width=None,veltype=None,outframe=None,gridfunction=None,convsupport=None,truncate=None,gwidth=None,jwidth=None,imsize=None,cell=None,phasecenter=None,ephemsrcname=None,pointingcolumn=None,restfreq=None,stokes=None,minweight=None,):

        """SD task: imaging for total power and spectral data
Keyword arguments:
infiles -- a list of names of input SD Measurementsets
        example: 'm100.PM01.ms'
                 ['m100.PM01.ms','m100.PM03.ms']; multiple MSes
outfile -- name of output image
        default: ''
        example: 'mySDimage.im'
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False (do NOT overwrite)
        example: if True, existing file will be overwritten
field -- select data by field IDs and names
                If field string is a non-negative integer, it is assumed to
                be a field index otherwise, it is assumed to be a 
                field name
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 3 or filed named 3C273)
                 For multiple MS input, a list of field strings can be used:
                 field = ['0~2','0~4'] (field ids 0-2 for the first MS and 0-4
                         for the second) 
                 field = '0~2' (field ids 0-2 for all input MSes)
        this selection is in addition to the other selections to data
spw -- select data by spectral window IDs/channels
       NOTE: channels de-selected here will contain all zeros if
       selected by the parameter mode subparameters.    
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
                 spw='0:5~61' (IF ID 0; channels 5 to 61; all channels)
                 spw='3:10~20;50~60' (select multiple channel ranges within IF ID 3)
                 spw='3:10~20,4:0~30' (select different channel ranges for IF IDs 3 and 4)
                 spw='1~4;6:15~48' (for channels 15 through 48 for IF IDs 1,2,3,4 and 6)
                 For multiple MS input, a list of spw strings can be used:
                 spw=['0','0~3'] (spw ids 0 for the first MS and 0-3 for the second)
                 spw='0~3' (spw ids 0-3 for all input MSes)
        this selection is in addition to the other selections to data
antenna -- select data by antenna names or IDs
           If antenna string is a non-negative integer, it is 
           assumed to be an antenna index, otherwise, it is
           considered an antenna name.
        default: '' (all baselines, i.e. all antenna in case of auto data)
        example: antenna='PM03'
                 For multiple MS input, a list of antenna strings can be used:
                 antenna=['5','6'] (antenna id5 for the first MS and 6 for the second)
                 antenna='5' (antenna index 5 for all input MSes)
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
                 For multiple MS input, a list of scan strings can be used:
                 scan=['0~100','10~200'] (scan ids 0-100 for the first MS
                 and 10-200 for the second)
                 scan='0~100 (scan ids 0-100 for all input MSes)
        this selection is in addition to the other selections to data
mode -- spectral gridding type
        options: 'channel', 'velocity', 'frequency'
        default: 'channel'
    >>> mode expandable parameters
       nchan -- Total number of channels in the output image.
           default: -1; Automatically selects enough channels to cover 
                    data selected by 'spw' consistent with 'start' and 'width'.
                    It is often easiest to leave nchan at the default value. 
           example: nchan=100
       start -- First channel, velocity, or frequency. 
                For mode='channel'; This selects the channel index number 
                from the MS (0 based) that you want to correspond to the
                first channel of the output cube. The output cube will be
                in frequency space with the first channel having the
                frequency of the MS channel selected by start.  start=0
                refers to the first channel in the first selected spw, even
                if that channel is de-selected in the spw parameter.
                Channels de-selected by the spw parameter will be filled with
                zeros if included by the start parameter. For example,
                spw=3~8:3~100 and start=2 will produce a cube that starts on
                the third channel (recall 0 based) of spw index 3, and the
                first channel will be blank.
           default: '' (the first input channel of first input spw)
           example: start=100 (mode='channel')
                    start='22.3GHz' (mode='frequency')
                    start='5.0km/s' (mode='velocity')
       width -- Output channel width
               For mode='channel', default=1; width>1 indicates channel averaging
               example: width=4.
               For mode= 'velocity' or 'frequency', default=''; width of
               first input channel, or more precisely, the difference 
               in frequencies between the first two selected channels. 
               -- For example if channels 1 and 3 are selected with spw, 
                then the default width will be the difference between their
                frequencies, and not the width of channel 1. 
               -- Similarly, if the selected data has uneven channel-spacing,
                 the default width will be picked from the first two selected
                 channels. In this case, please specify the desired width.
               When specifying the width, one must give units
               examples: width='1.0km/s', or width='24.2kHz'.
               Setting width>0 gives channels of increasing frequency for 
               mode='frequency', and increasing velocity for mode='velocity'.
       veltype -- Velocity reference frame of output image
           Options: 'radio','optical','true','relativistic'
           default: 'radio'
outframe -- velocity reference frame of output image
        Options: '','LSRK','LSRD','BARY','GEO','TOPO','GALACTO',
                 'LGROUP','CMB'
        default: ''; same as input data or 'LSRK' for multiple-MS inputs
        example: frame='bary' for Barycentric frame 
gridfunction -- gridding function for imaging
        options: 'BOX' (Box-car), 'SF' (Spheroidal), 
                 'PB' (Primary-beam), 'GAUSS' (Gaussian),
                 'GJINC' (Gaussian*Jinc)
        default: 'BOX'
        example: 'SF'
    >>> gridfunction expandable parameter:
       convsupport -- convolution support for 'SF' 
           default: -1 (use default for each gridfunction)
           example: 3
       truncate -- truncattion radius of convolution kernel.
                   effective only for 'GAUSS' and 'GJINC'.
           default: '-1' (use default for each gridfunction)
           example: 3, '20arcsec', '3pixel'
       gwidth -- HWHM for gaussian. Effective only for 
                 'GAUSS' and 'GJINC'.
           default: '-1' (use default for each gridfunction)
           example: 3, '20arcsec', '3pixel'
       jwidth -- Width of jinc function. Effective only for 
                 'GJINC'.
           default: '-1' (use default for each gridfunction)
           example: 3, '20arcsec', '3pixel'
imsize -- x and y image size in pixels, symmetric for single value
        default: [] (=cover all pointings in MS)
        example: imsize=200 (equivalent to [200,200])
cell -- x and y cell size. default unit arcmin
        default: '' (= 1/3 of FWHM of primary beam)
        example: cell=['0.2arcmin, 0.2arcmin']
                 cell='0.2arcmin' (equivalent to example above)
phasecenter -- image phase center: direction measure or field ID
        default: '' (= the center of pointing directions in 
                     POINTING table of infiles)
        example: 6 (field id), 'J2000 13h44m00 -17d02m00',
                 'AZEL -123d48m29 15d41m41'
ephemsrcname -- ephemeris source name for moving source (solar sytem objects)
        default: '' (none)
        if the source name in the data matches one of the solar system
        objects known by CASA, the imaging realigns the data by
        correcting shifts of the source during observation,
        so that the source appears to be fixed in the image.
        examples: 'MERCURY', 'VENUS', 'MARS', 'JUPITER', 'SATURN',
                  'URANUS', 'NEPUTUNE', 'PLUTO', 'SUN', 'MOON'
pointingcolumn -- pointing data column to use
        option: 'direction', 'target', 'pointing_offset', 'source_offset', encoder' 
        default: 'direction'
restfreq -- specify rest frequency to use for output image
        default: '' (refer input data)
        example: 1.0e11, '100GHz'
stokes -- stokes parameters or polarization types to image
        default: '' (use all polarizations)
        example: 'XX'
minweight -- Minimum weight ratio to the median of weight used in 
             weight correction and weight based masking
        default: 0.1
        example: minweight = 0.


-----------------
Gridding Kernel
-----------------
The parameter gridfunction sets gridding function (convolution kernel)
for imaging. Currently, the task supports 'BOX' (Box-car), 'SF' (Prolate
Spheroidal Wave Function), 'GAUSS' (Gaussian), 'GJINC' (Gaussian*Jinc),
where Jinc(x) = J_1(pi*x/c)/(pi*x/c) with a first order Bessel function
J_1, and 'PB' (Primary Beam). For 'PB', correct antenna informations
should be included in input file.

There are four subparameters for gridfunction: convsupport, truncate, 
gwidth, and jwidth. The convsupport is an integer specifying cut-off 
radius for 'SF' in units of pixel. By default (convsupport=-1), 
the cut-off radius is set to 3 pixels. The truncate is a cut-off 
radius for 'GAUSS' or 'GJINC'. It accepts integer, float, and 
string values of numeric plus unit. Allowed units are angular 
units such as 'deg', 'arcmin', 'arcsec', and 'pixel'. Default unit 
is 'pixel' so that string without unit or numerical values (integer 
or float) will be interpreted as radius in pixel. Default value 
for truncate, which is used when negative radius is set, is 3*HWHM 
for 'GAUSS' and radius at first null for 'GJINC'. The gwidth is 
the HWHM of gaussian for 'GAUSS' and 'GJINC'. Default value is 
sqrt(log(2)) pixel for 'GAUSS' and 2.52*sqrt(log(2)) pixel for 
'GJINC'. The jwidth specifies width of the jinc function (parameter 
'c' in the definition above). Default is 1.55 pixel. Both gwidth 
jwidth allows integer, float, or string of numeric plus unit. 
Default values for gwidth and jwidth are taken from Mangum et al. 
(2007). Formula for 'GAUSS' and 'GJINC' are taken from Table 1 in 
the paper, and are written as below using gwidth and jwidth: 

   GAUSS: exp[-log(2)*(|r|/gwidth)**2]

   GJINC: J_1(pi*|r|/jwidth)/(pi*|r|/jwidth)
             * exp[-log(2)*(|r|/gwidth)^2]  


Reference: Mangum, et al. 2007, A&A, 474, 679-687 

--------------------
Mask in Output Image
--------------------
The parameter minweight defines a threshold of weight values 
to mask. The pixels in outfile whose weight is smaller than
minweight*median(weight) are masked out. The task also creates
a weight image with the name outfile.weight.

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdimaging'
        myf['taskname'] = 'sdimaging'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infiles'] = infiles = myf['infiles']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['scan'] = scan = myf['scan']
            myparams['mode'] = mode = myf['mode']
            myparams['nchan'] = nchan = myf['nchan']
            myparams['start'] = start = myf['start']
            myparams['width'] = width = myf['width']
            myparams['veltype'] = veltype = myf['veltype']
            myparams['outframe'] = outframe = myf['outframe']
            myparams['gridfunction'] = gridfunction = myf['gridfunction']
            myparams['convsupport'] = convsupport = myf['convsupport']
            myparams['truncate'] = truncate = myf['truncate']
            myparams['gwidth'] = gwidth = myf['gwidth']
            myparams['jwidth'] = jwidth = myf['jwidth']
            myparams['imsize'] = imsize = myf['imsize']
            myparams['cell'] = cell = myf['cell']
            myparams['phasecenter'] = phasecenter = myf['phasecenter']
            myparams['ephemsrcname'] = ephemsrcname = myf['ephemsrcname']
            myparams['pointingcolumn'] = pointingcolumn = myf['pointingcolumn']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['minweight'] = minweight = myf['minweight']

        if type(infiles)==str: infiles=[infiles]
        if type(imsize)==int: imsize=[imsize]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infiles'] = infiles
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['antenna'] = antenna
        mytmp['scan'] = scan
        mytmp['mode'] = mode
        mytmp['nchan'] = nchan
        mytmp['start'] = start
        mytmp['width'] = width
        mytmp['veltype'] = veltype
        mytmp['outframe'] = outframe
        mytmp['gridfunction'] = gridfunction
        mytmp['convsupport'] = convsupport
        mytmp['truncate'] = truncate
        mytmp['gwidth'] = gwidth
        mytmp['jwidth'] = jwidth
        mytmp['imsize'] = imsize
        mytmp['cell'] = cell
        mytmp['phasecenter'] = phasecenter
        mytmp['ephemsrcname'] = ephemsrcname
        mytmp['pointingcolumn'] = pointingcolumn
        mytmp['restfreq'] = restfreq
        mytmp['stokes'] = stokes
        mytmp['minweight'] = minweight
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdimaging.xml')

        casalog.origin('sdimaging')
        if not trec.has_key('sdimaging') or not casac.utils().verify(mytmp, trec['sdimaging']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdimaging           #####')
          casalog.post('')
          result = sdimaging(infiles, outfile, overwrite, field, spw, antenna, scan, mode, nchan, start, width, veltype, outframe, gridfunction, convsupport, truncate, gwidth, jwidth, imsize, cell, phasecenter, ephemsrcname, pointingcolumn, restfreq, stokes, minweight)
          casalog.post('')
          casalog.post('##### End Task: sdimaging           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdimaging', 'sdimaging.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdimaging'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdimaging', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infiles']  = ['']
        a['outfile']  = ''
        a['overwrite']  = False
        a['field']  = ''
        a['spw']  = ''
        a['antenna']  = ''
        a['scan']  = ''
        a['mode']  = 'channel'
        a['outframe']  = ''
        a['gridfunction']  = 'BOX'
        a['imsize']  = [
            ]
        a['cell']  = ''
        a['phasecenter']  = ''
        a['ephemsrcname']  = ''
        a['pointingcolumn']  = 'direction'
        a['restfreq']  = ''
        a['stokes']  = ''
        a['minweight']  = 0.1

        a['mode'] = {
                    0:odict([{'value':'channel'}, {'nchan':-1}, {'start':''}, {'width':''}]), 
                    1:odict([{'value':'frequency'}, {'nchan':-1}, {'start':''}, {'width':''}]), 
                    2:odict([{'value':'velocity'}, {'nchan':-1}, {'start':''}, {'width':''}, {'veltype':'radio'}])}
        a['gridfunction'] = {
                    0:{'value':'BOX'}, 
                    1:{'value':'box'}, 
                    2:odict([{'value':'SF'}, {'convsupport':-1}]), 
                    3:odict([{'value':'sf'}, {'convsupport':-1}]), 
                    4:{'value':'PB'}, 
                    5:{'value':'pb'}, 
                    6:odict([{'value':'GAUSS'}, {'truncate':-1}, {'gwidth':-1}]), 
                    7:odict([{'value':'gauss'}, {'truncate':-1}, {'gwidth':-1}]), 
                    8:odict([{'value':'GJINC'}, {'truncate':-1}, {'gwidth':-1}, {'jwidth':-1}]), 
                    9:odict([{'value':'gjinc'}, {'truncate':-1}, {'gwidth':-1}, {'jwidth':-1}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdimaging', subkey=None):
        desc={'sdimaging': 'SD task: imaging for total power and spectral data',
               'infiles': 'a list of names of input SD Measurementsets (only MS is allowed for this task)',
               'outfile': 'name of output image',
               'overwrite': 'overwrite the output file if already exists [True, False]',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'antenna': 'select data by antenna names or IDs, e.g, \'PM03\' (\'\' = all antennas)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'mode': 'spectral gridding type',
               'nchan': ' number of channels (planes) in output image (-1=all)',
               'start': 'start of output spectral dimension, e.g. \'0\', \'110GHz\', \'-20km/s\'',
               'width': 'width of output spectral channels',
               'veltype': 'velocity definition',
               'outframe': 'velocity frame of output image (''=current frame or LSRK for multiple-MS inputs)',
               'gridfunction': 'gridding function for imaging (see description in help)',
               'convsupport': 'convolution support for gridding',
               'truncate': 'truncation radius for gridding',
               'gwidth': 'HWHM for gaussian',
               'jwidth': 'c-parameter for jinc function',
               'imsize': 'x and y image size in pixels, e.g., [64,64]. Single value: same for both spatial axes ([] = number of pixels to cover whole pointings in MSes)',
               'cell': 'x and y cell size, (e.g., [\'8arcsec\',\'8arcsec\']. default unit arcmin. ('' = 1/3 of FWHM of primary beam)',
               'phasecenter': 'image center direction: position or field index, e.g., \'J2000 17:30:15.0 -25.30.00.0\'. ('' = the center of pointing directions in MSes)',
               'ephemsrcname': 'ephemeris source name, e.g. \'MARS\'',
               'pointingcolumn': 'pointing data column to use',
               'restfreq': 'rest frequency to assign to image, e.g., \'114.5GHz\'',
               'stokes': 'stokes parameters or polarization types to image, e.g. \'I\', \'XX\'',
               'minweight': 'Minimum weight ratio to the median of weight used in weight correction and weight beased masking',

              }

#
# Set subfields defaults if needed
#
        if(subkey == 'channel'):
          desc['start'] = 'Begin the output cube at the frequency of this channel in the MS'
        if(subkey == 'channel'):
          desc['width'] = 'Width of output channel relative to MS channel (# to average)(\'\' = 1 channel) '
        if(subkey == 'frequency'):
          desc['start'] = 'Frequency of first channel: e.g. \'1.4GHz\' (\'\'= first channel in first SpW of MS)'
        if(subkey == 'frequency'):
          desc['width'] = 'Channel width: e.g \'1.0kHz\'(\'\'=width of first channel in first SpW of MS)'
        if(subkey == 'velocity'):
          desc['start'] = 'Velocity of first channel: e.g \'0.0km/s\'(\'\'=first channel in first SpW of MS)'
        if(subkey == 'velocity'):
          desc['width'] = 'Channel width e.g \'-1.0km/s\' (\'\'=width of first channel in first SpW of MS)'

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infiles']  = ['']
        a['outfile']  = ''
        a['overwrite']  = False
        a['field']  = ''
        a['spw']  = ''
        a['antenna']  = ''
        a['scan']  = ''
        a['mode']  = 'channel'
        a['nchan']  = -1
        a['start']  = 0
        a['width']  = 1
        a['veltype']  = 'radio'
        a['outframe']  = ''
        a['gridfunction']  = 'BOX'
        a['convsupport']  = -1
        a['truncate']  = -1
        a['gwidth']  = -1
        a['jwidth']  = -1
        a['imsize']  = [
            ]
        a['cell']  = ''
        a['phasecenter']  = ''
        a['ephemsrcname']  = ''
        a['pointingcolumn']  = 'direction'
        a['restfreq']  = ''
        a['stokes']  = ''
        a['minweight']  = 0.1

        if a.has_key(paramname) :
	      return a[paramname]
sdimaging_pg = sdimaging_pg_()
