#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdmath import sdmath
from task_sdmath import casalog

class sdmath_pg_:
    __name__ = "sdmath"

    def __init__(self) :
       self.__bases__ = (sdmath_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infiles=None,expr=None,varnames=None,antenna=None,fluxunit=None,telescopeparam=None,field=None,spw=None,scan=None,pol=None,outfile=None,outform=None,overwrite=None,):

        """ASAP SD task for simple arithmetic of spectra
Keyword arguments:
infiles -- a list of names of input SD datasets
        The file names will automatically replace the phrases 
        IN0, IN1, ... in expr parameter.
expr -- mathematical expression using scantables 
varnames -- a python dictionary of variables in expr and their values.
        Keys must be coincide with variables used in expr.
        Values are substituted in each value in expr.
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
fluxunit -- units for line flux
        options: 'K','Jy',''
        default: '' (keep current fluxunit in data)
        WARNING: For GBT data, see description below.
    >>> fluxunit expandable parameter
         telescopeparam -- parameters of telescope for flux conversion
                 options: (str) name or (list) list of gain info
                 default: '' (none set)
                 example: if telescopeparam='', it tries to get the telescope
                          name from the data.
                          Full antenna parameters (diameter,ap.eff.) known
                          to ASAP are
                          'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                          'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                          to 'K' first then convert to a new fluxunit.
                          telescopeparam=[104.9,0.43] diameter(m), ap.eff.
                          telescopeparam=[0.743] gain in Jy/K
                          telescopeparam='FIX' to change default fluxunit
                          see description below
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)
       NOTE this task only supports IF ID selction and ignores channel
       selection.
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
outfile -- name of output file
        default: '' (must be specified)
outform -- output file format
        options: 'ASAP','MS2', 'ASCII','SDFITS'
        default: 'ASAP'
        NOTE the ASAP format is easiest for further sd
        processing; use MS2 for CASA imaging.
        If ASCII, then will append some stuff to
        the outfile name
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'


DESCRIPTION:

Task sdmath execute a simple arithmetic (i.e., subtraction, addition, 
multiplication, and division) expression for single dish spectra.
The spectral data file can be any of the formats supported by
ASAP (scantable, MS, rpfits, and SDFITS). In the expression, 
these file names should be put inside of single or double quotes.

You can use variables in the expression. If you want to use, you 
must define varnames dictionary. Name of variables should be simple, 
e.g. V0, V1, etc., to avoid unexpected error. Keys of varnames must 
be name of variables that you used in the expression, and their 
values will be substituted for variables in the expression. Allowed 
type for the value is numerical values, one- or two-dimensional lists 
(Python list or numpy.ndarray), and filename strings that indicate 
spectral data or ASCII text, which is space-separated list of 
numerical values consisting of adequate number of rows and columns. 
In case you give a list of file names in infiles, they are 
automatically referred to as IN0, IN1, etc. in expr and you can not 
use IN0, IN1, etc. as variable names in varnames.

The fluxunit can be set, otherwise, the current settings of the first 
spectral data in the expression are used.  
Other selections (e.g. scan No, . IF, Pol) also apply to all 
the spectral data in the expression, so if any of the data does
not contains selection, the task will produce no output. 
       
WARNING for the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

Example:
# do on-off/off calculation
expr='("orion_on_data.asap"-"orion_off_data.asap")/"orion_off_data.asap"
outfile='orion_cal.asap'
sdmath()

# do on-off/off calculation (using infiles)
infiles = ["orion_on_data.asap", "orion_off_data.asap"]
expr='(IN0-IN1)/IN1'
outfile='orion_cal.asap'
sdmath()

# do on-off/off calculation using varnames
varnames={} (this can be skipped if you executed inp(sdmath) or 
             default(sdmath).)
varnames['V0']="orion_on_data.asap"
varnames['V1']="orion_off_data.asap"
varnames['V2']=1.0
expr='V0/V1-V2'
outfile='orion_cal.asap'
sdmath()

# do on-off/off calculation using varnames (in pythonic way)
sdmath(varnames={'V0':'orion_on_data.asap','V1':'orion_off_data.asap',
        'V2':1.0}, expr='V0/V1-V2', outfile='orion_cal.asap')

# interpretation of ASCII file value for varnames
If the contents of input ASCII file is shown as, 

   0.5 0.3 0.2
   1.0 0.2 0.9

it is interpreted as a list [[0.5, 0.3, 0.2],[1.0, 0.2, 0.9]]. 

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdmath'
        myf['taskname'] = 'sdmath'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infiles'] = infiles = myf['infiles']
            myparams['expr'] = expr = myf['expr']
            myparams['varnames'] = varnames = myf['varnames']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['fluxunit'] = fluxunit = myf['fluxunit']
            myparams['telescopeparam'] = telescopeparam = myf['telescopeparam']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['scan'] = scan = myf['scan']
            myparams['pol'] = pol = myf['pol']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(infiles)==str: infiles=[infiles]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infiles'] = infiles
        mytmp['expr'] = expr
        mytmp['varnames'] = varnames
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparam'] = telescopeparam
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdmath.xml')

        casalog.origin('sdmath')
        if not trec.has_key('sdmath') or not casac.utils().verify(mytmp, trec['sdmath']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdmath           #####')
          casalog.post('')
          result = sdmath(infiles, expr, varnames, antenna, fluxunit, telescopeparam, field, spw, scan, pol, outfile, outform, overwrite)
          casalog.post('')
          casalog.post('##### End Task: sdmath           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdmath', 'sdmath.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdmath'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdmath', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infiles']  = ['']
        a['expr']  = ''
        a['varnames']  = {}
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparam':''}]), 
                    2:odict([{'value':'k'}, {'telescopeparam':''}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparam':''}]), 
                    4:odict([{'value':'jy'}, {'telescopeparam':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdmath', subkey=None):
        desc={'sdmath': 'ASAP SD task for simple arithmetic of spectra',
               'infiles': 'a list of names of input SD datasets',
               'expr': 'mathematical expression using spectra',
               'varnames': 'dictionary of variables and their values used in expr',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'fluxunit': 'units of the flux (\'\'=current)',
               'telescopeparam': 'parameters of telescope for flux conversion (see examples in help)',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g. \'0,1\' (\'\'=all)',
               'outfile': 'name of output file (must be specified)',
               'outform': 'output file format (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists [True, False]',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infiles']  = ['']
        a['expr']  = ''
        a['varnames']  = {}
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparam']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
sdmath_pg = sdmath_pg_()
