#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdscale import sdscale
class sdscale_cli_:
    __name__ = "sdscale"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdscale_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'antenna':None, 'factor':None, 'scaletsys':None, 'outfile':None, 'overwrite':None, 'verbose':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, antenna=None, factor=None, scaletsys=None, outfile=None, overwrite=None, verbose=None, ):

        """Scale the sd spectra

	Detailed Description: 

Task sdscale performs scaling of single-dish spectra by scaling
factor given by parameter named factor.
By setting scaletsys = True, associated Tsys is also scaled.
  
	Arguments :
		infile:	name of input SD dataset
		   Default Value: 

		antenna:	select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)
		   Default Value: 0

		factor:	scaling factor (float or float list)
		   Default Value: 1.0

		scaletsys:	scaling of associated Tsys
		   Default Value: True

		outfile:	name of output file (See a WARNING in help)
		   Default Value: 

		overwrite:	overwrite the output file if already exists
		   Default Value: False

		verbose:	Print verbose log output
		   Default Value: True

	Returns: void

	Example :

------------------
Keyword arguments
------------------
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
factor -- scaling factor. float, one- or two-dimensional float list,
          or filename storing scaling factor are acceptable
       default: 1.0 (no scaling)
       example: see description below
scaletsys -- scaling of associated Tsys
          options: (bool) True, False
          default: True
outfile -- name of output file
        default: outfile='' (<infile>_scaled<factor>)
        example: 'scaled.asap'
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'
verbose -- Print verbose log messages. If True, Tsys values before
           (and after) scaling are printed to logger.
          options: (bool) True, False
          default: True

-----------
DESCRIPTION
-----------
Task sdscale performs scaling of single-dish spectra.
Associated Tsys is also scaled if scaletsys is True.
Tsys informations are written in the casalogger and they are 
automatically stored in 'casapy.log'. 
The infile can be any of ASAP, MS, SDFITS, or RPFITS format.
If outfile name is given or outfile=''(default), the scaled data 
is written to a new file with the same format as the input data 
(Note: in case of the RPFITS format input data, it will be written 
to SDFITS format).

The scaling factor, factor, accepts both scalar type and list 
type value. The list must be one or two dimensional. If factor is 
one dimensional, its length must coincide with a number of spectral 
channel. If factor is two dimensional, its shape must be 
(n, 1) or (n, m), where n is a number of spectrum, while m is a 
number of channel for each spcetum. m can be variable for each 
spectrum. In addition, the factor can be an ASCII filename that 
stores a space-separated list of scaling factor consisting of 
adequate number of rows and columns. For example, if the contents 
of input ASCII file is shown as, 

   0.5 0.3 0.2
   1.0 0.2 0.9

it is interpreted as a list [[0.5, 0.3, 0.2],[1.0, 0.2, 0.9]]. 

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdscale'
        self.__globals__['taskname'] = 'sdscale'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['factor'] = factor = self.parameters['factor']
            myparams['scaletsys'] = scaletsys = self.parameters['scaletsys']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']
            myparams['verbose'] = verbose = self.parameters['verbose']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['factor'] = factor
        mytmp['scaletsys'] = scaletsys
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['verbose'] = verbose
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'sdscale.xml')

        casalog.origin('sdscale')
	try :
          #if not trec.has_key('sdscale') or not casac.casac.utils().verify(mytmp, trec['sdscale']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdscale'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdscale', 'sdscale.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdscale'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdscale(infile, antenna, factor, scaletsys, outfile, overwrite, verbose)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdscale'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdscale', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['factor']  = 1.0
        a['scaletsys']  = True
        a['outfile']  = ''
        a['overwrite']  = False
        a['verbose']  = True


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdscale', subkey=None):
        desc={'sdscale': 'Scale the sd spectra',
               'infile': 'name of input SD dataset',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'factor': 'scaling factor (float or float list)',
               'scaletsys': 'scaling of associated Tsys [True, False]',
               'outfile': 'name of output file (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists [True, False]',
               'verbose': 'Print verbose log output [True, False]',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['factor']  = 1.0
        a['scaletsys']  = True
        a['outfile']  = ''
        a['overwrite']  = False
        a['verbose']  = True

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if a.has_key(paramname) :
	      return a[paramname]
sdscale_cli = sdscale_cli_()
