#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdscale import sdscale
from task_sdscale import casalog

class sdscale_pg_:
    __name__ = "sdscale"

    def __init__(self) :
       self.__bases__ = (sdscale_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,factor=None,scaletsys=None,outfile=None,overwrite=None,verbose=None,):

        """Scale the sd spectra
------------------
Keyword arguments
------------------
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
factor -- scaling factor. float, one- or two-dimensional float list,
          or filename storing scaling factor are acceptable
       default: 1.0 (no scaling)
       example: see description below
scaletsys -- scaling of associated Tsys
          options: (bool) True, False
          default: True
outfile -- name of output file
        default: outfile='' (<infile>_scaled<factor>)
        example: 'scaled.asap'
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'
verbose -- Print verbose log messages. If True, Tsys values before
           (and after) scaling are printed to logger.
          options: (bool) True, False
          default: True

-----------
DESCRIPTION
-----------
Task sdscale performs scaling of single-dish spectra.
Associated Tsys is also scaled if scaletsys is True.
Tsys informations are written in the casalogger and they are 
automatically stored in 'casapy.log'. 
The infile can be any of ASAP, MS, SDFITS, or RPFITS format.
If outfile name is given or outfile=''(default), the scaled data 
is written to a new file with the same format as the input data 
(Note: in case of the RPFITS format input data, it will be written 
to SDFITS format).

The scaling factor, factor, accepts both scalar type and list 
type value. The list must be one or two dimensional. If factor is 
one dimensional, its length must coincide with a number of spectral 
channel. If factor is two dimensional, its shape must be 
(n, 1) or (n, m), where n is a number of spectrum, while m is a 
number of channel for each spcetum. m can be variable for each 
spectrum. In addition, the factor can be an ASCII filename that 
stores a space-separated list of scaling factor consisting of 
adequate number of rows and columns. For example, if the contents 
of input ASCII file is shown as, 

   0.5 0.3 0.2
   1.0 0.2 0.9

it is interpreted as a list [[0.5, 0.3, 0.2],[1.0, 0.2, 0.9]]. 

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdscale'
        myf['taskname'] = 'sdscale'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['factor'] = factor = myf['factor']
            myparams['scaletsys'] = scaletsys = myf['scaletsys']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['verbose'] = verbose = myf['verbose']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['factor'] = factor
        mytmp['scaletsys'] = scaletsys
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['verbose'] = verbose
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdscale.xml')

        casalog.origin('sdscale')
        if not trec.has_key('sdscale') or not casac.utils().verify(mytmp, trec['sdscale']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdscale           #####')
          casalog.post('')
          result = sdscale(infile, antenna, factor, scaletsys, outfile, overwrite, verbose)
          casalog.post('')
          casalog.post('##### End Task: sdscale           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdscale', 'sdscale.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdscale'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdscale', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['factor']  = 1.0
        a['scaletsys']  = True
        a['outfile']  = ''
        a['overwrite']  = False
        a['verbose']  = True


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdscale', subkey=None):
        desc={'sdscale': 'Scale the sd spectra',
               'infile': 'name of input SD dataset',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'factor': 'scaling factor (float or float list)',
               'scaletsys': 'scaling of associated Tsys',
               'outfile': 'name of output file (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists',
               'verbose': 'Print verbose log output',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['factor']  = 1.0
        a['scaletsys']  = True
        a['outfile']  = ''
        a['overwrite']  = False
        a['verbose']  = True

        if a.has_key(paramname) :
	      return a[paramname]
sdscale_pg = sdscale_pg_()
