#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_simanalyze import simanalyze
from task_simanalyze import casalog

class simanalyze_pg_:
    __name__ = "simanalyze"

    def __init__(self) :
       self.__bases__ = (simanalyze_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, project=None,image=None,imagename=None,skymodel=None,vis=None,modelimage=None,imsize=None,imdirection=None,cell=None,interactive=None,niter=None,threshold=None,weighting=None,mask=None,outertaper=None,pbcor=None,stokes=None,featherimage=None,analyze=None,showuv=None,showpsf=None,showmodel=None,showconvolved=None,showclean=None,showresidual=None,showdifference=None,showfidelity=None,graphics=None,verbose=None,overwrite=None,dryrun=None,logfile=None,):

        """image and analyze measurement sets created with simobserve
      * "project" needs to be the directory of results generated by running 
        simobserve or simalma. In particular $project/$project.skymodel 
        will be required in order to compare output and input images.

    -------------------------------
    mode image=True:
      * One should input one or more simulated MSs using the "vis" parameter.  
        These can include a total power MS. 
        Simanalyze will grid any total power MS, 
        clean (invert and deconvolve) any interferometric MSs, 
        and feather the results. 

      * the "vis" parameter:
        - example: single MS: vis="mysim.alma.out03.ms"
        - example: multiple MSs: vis=["mysim.alma.out03.ms","mysim.aca.tp.ms"]
        - one can use '$project' and let the task automatically replace it with
          the project name, e.g., vis="$project.noisy.ms,$project.noisy.sd.ms".
          However, note that if you created measurement set(s) using simobserve,
          MS names will include the configuration, e.g. 
          $project.alma_out20.noisy.ms
        - setting "vis" to "default" will find and attempt to image
          all measurement sets (interferometric and single dish) in the 
          project directory

      * Sometimes it is preferable to grid the single dish MS using the 
        sdimaging task for more control.  In that case one can input 
        the resulting single dish imaging under "featherimage", only 
        put interferometric MSs in "vis", and simanalyze  will clean the 
        interferometric and feather with your "featherimage".

      * Sometimes it is preferable to use a low resolution (single dish or 
        synthesis) image as a prior model during clean deconvolution
        of a higher resolution interferometric MS.  That is accomplished 
        by putting the low-resolution image in "modelimage" and the MS
        to be deconvolved in "vis". NOTE: This is not the original skymodel
        that was used in simobserve or simalma.  It is recommended to 
        leave this blank unless the user is familiar with using a prior 
        in clean deconvolution. (see casaguides) NOTE 2: modelimage will 
        not be used if the MS to be imaged is total power.
       
      * uses Cotton-Schwab clean for single fields and Mosaic gridding
        for multiple fields (with Clark PSF calculation in minor cycles).

      * interactive clean or use of more parameters than the subset 
        visible here are available by simply running the clean task directly, 
        then using simanalyze in the mode image=False (see below).

      * if graphics are turned on, this step will display the clean image 
        and residual image

      * the "mask" parameter:
        Specification of cleanbox(es), mask image(s), primary beam
        coverage level, and/or region(s) to be used for cleaning.
        clean tends to perform better, and is less likely to diverge, if 
        the clean component placement is limited by a mask to where real 
        emission is expected to be.  e.g. pixel ranges mask=[110,110,150,145],
        filename of mask image mask='myimage.mask', or a file with mask 
        regions --  see help for the clean task for more information.

    -------------------------------
    mode image=False:
      * Sometimes the user has created a synthesized image themselves, 
        most likely using the clean task, perhaps along with 
        sdimaging and feather, or a previous call to simanalyze with image=True 
      * The user should input that simulated image as "imagename".  
        It will have suffix .image if created by clean, simanalyze, or simalma
      * simanalyze will attempt to find an appropriate skymodel image - 
        this is the *.skymodel image created by simobserve or simalma, 
        the (optionally rescaled) original sky model, which was used 
        to create the measurement set.
        simanalyze will look in the project directory, but if there are 
        multuple skymodels present it may not find the right one, so the
        "skymodel" parameter allows explicit specification.

    -------------------------------
    mode analyze=True is used to create an image of the difference between 
        the input skymodel and the simulated output image (whether that output
        image is being generated in the same call to simanalyze, with 
        image=True, or has already been generated, and simanalyze is being 
        called with image=False).

    showuv -- display uv coverage
    showpsf -- display synthesized (dirty) beam (ignored in single dish simulation)
    showmodel -- display sky model at original resolution
    showconvolved -- display sky model convolved with output beam 
    showclean -- display the synthesized image
    showresidual -- display the clean residual image (ignored in single dish simulation)
    showdifference -- display difference between output cleaned image and 
         input model sky image convolved with output clean beam
    showfidelity -- display fidelity image
         fidelity = abs(input) / max[ abs(input-output), 0.7*rms(output) ]
    

    Note that the RMS is calculated in the lower quarter of the image.  
         This is likely not the best choice, so you are encouraged to 
         measure RMS yourself in an off-source region using the viewer.

    dryrun=True is an advanced technical mode only useful for interferometric 
         (not single dish) data. 

    -------------------------------
    Output produced: (not all will always exist, depending on input parameters)
    To support different runs with different arrays, the names have the
    configuration name from antennalist appended.
    -------------------------------
    project.[cfg].skymodel.flat.regrid.conv = input sky regridded to match 
         the output image, and convolved with the output clean beam

    project.[cfg].image = synthesized image
    project.[cfg].flux.pbcoverage = primary beam correction for mosaic image
    project.[cfg].residual = residual image after cleaning
    project.[cfg].clean.last = parameter file of what parameters were used in
          the clean task
    project.[cfg].psf = synthesized (dirty) beam calculated from weighted uv 
          distribution
    project.[cfg].image.png = diagnostic figure of clean image and residual
	    
    project.[cfg].fidelity = fidelity image
    project.[cfg].analysis.png = diagnostic figure of difference and fidelity
	    
    project.[cfg].simanalyze.last = saved input parameters for simanalyze task

    -------------------------------
    Please see http://casaguides.nrao.edu, and contact the CASA helpdesk 
    with questions.
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'simanalyze'
        myf['taskname'] = 'simanalyze'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['project'] = project = myf['project']
            myparams['image'] = image = myf['image']
            myparams['imagename'] = imagename = myf['imagename']
            myparams['skymodel'] = skymodel = myf['skymodel']
            myparams['vis'] = vis = myf['vis']
            myparams['modelimage'] = modelimage = myf['modelimage']
            myparams['imsize'] = imsize = myf['imsize']
            myparams['imdirection'] = imdirection = myf['imdirection']
            myparams['cell'] = cell = myf['cell']
            myparams['interactive'] = interactive = myf['interactive']
            myparams['niter'] = niter = myf['niter']
            myparams['threshold'] = threshold = myf['threshold']
            myparams['weighting'] = weighting = myf['weighting']
            myparams['mask'] = mask = myf['mask']
            myparams['outertaper'] = outertaper = myf['outertaper']
            myparams['pbcor'] = pbcor = myf['pbcor']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['featherimage'] = featherimage = myf['featherimage']
            myparams['analyze'] = analyze = myf['analyze']
            myparams['showuv'] = showuv = myf['showuv']
            myparams['showpsf'] = showpsf = myf['showpsf']
            myparams['showmodel'] = showmodel = myf['showmodel']
            myparams['showconvolved'] = showconvolved = myf['showconvolved']
            myparams['showclean'] = showclean = myf['showclean']
            myparams['showresidual'] = showresidual = myf['showresidual']
            myparams['showdifference'] = showdifference = myf['showdifference']
            myparams['showfidelity'] = showfidelity = myf['showfidelity']
            myparams['graphics'] = graphics = myf['graphics']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['dryrun'] = dryrun = myf['dryrun']
            myparams['logfile'] = logfile = myf['logfile']

        if type(imsize)==int: imsize=[imsize]
        if type(outertaper)==str: outertaper=[outertaper]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['project'] = project
        mytmp['image'] = image
        mytmp['imagename'] = imagename
        mytmp['skymodel'] = skymodel
        mytmp['vis'] = vis
        mytmp['modelimage'] = modelimage
        mytmp['imsize'] = imsize
        mytmp['imdirection'] = imdirection
        mytmp['cell'] = cell
        mytmp['interactive'] = interactive
        mytmp['niter'] = niter
        mytmp['threshold'] = threshold
        mytmp['weighting'] = weighting
        mytmp['mask'] = mask
        mytmp['outertaper'] = outertaper
        mytmp['pbcor'] = pbcor
        mytmp['stokes'] = stokes
        mytmp['featherimage'] = featherimage
        mytmp['analyze'] = analyze
        mytmp['showuv'] = showuv
        mytmp['showpsf'] = showpsf
        mytmp['showmodel'] = showmodel
        mytmp['showconvolved'] = showconvolved
        mytmp['showclean'] = showclean
        mytmp['showresidual'] = showresidual
        mytmp['showdifference'] = showdifference
        mytmp['showfidelity'] = showfidelity
        mytmp['graphics'] = graphics
        mytmp['verbose'] = verbose
        mytmp['overwrite'] = overwrite
        mytmp['dryrun'] = dryrun
        mytmp['logfile'] = logfile
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'simanalyze.xml')

        casalog.origin('simanalyze')
        if not trec.has_key('simanalyze') or not casac.utils().verify(mytmp, trec['simanalyze']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: simanalyze           #####')
          casalog.post('')
          result = simanalyze(project, image, imagename, skymodel, vis, modelimage, imsize, imdirection, cell, interactive, niter, threshold, weighting, mask, outertaper, pbcor, stokes, featherimage, analyze, showuv, showpsf, showmodel, showconvolved, showclean, showresidual, showdifference, showfidelity, graphics, verbose, overwrite, dryrun, logfile)
          casalog.post('')
          casalog.post('##### End Task: simanalyze           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('simanalyze', 'simanalyze.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'simanalyze'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('simanalyze', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['project']  = 'sim'
        a['image']  = True
        a['analyze']  = False
        a['graphics']  = 'both'
        a['verbose']  = False
        a['overwrite']  = True
        a['dryrun']  = False
        a['logfile']  = ''

        a['image'] = {
                    0:odict([{'value':True}, {'vis':'default'}, {'modelimage':''}, {'imsize':0}, {'imdirection':''}, {'cell':''}, {'interactive':False}, {'niter':0}, {'threshold':'0.1mJy'}, {'weighting':'natural'}, {'mask':[]}, {'outertaper':[]}, {'pbcor':True}, {'stokes':'I'}, {'featherimage':''}]), 
                    1:odict([{'value':False}, {'imagename':'default'}, {'skymodel':''}])}
        a['analyze'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'showuv':True}, {'showpsf':True}, {'showmodel':True}, {'showconvolved':False}, {'showclean':True}, {'showresidual':False}, {'showdifference':True}, {'showfidelity':True}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='simanalyze', subkey=None):
        desc={'simanalyze': 'image and analyze measurement sets created with simobserve',
               'project': 'root prefix for output file names',
               'image': '(re)image $project.*.ms to $project.image',
               'imagename': 'simulation output image to analyze (default = first $project/*.image found)',
               'skymodel': 'skymodel image to analyze (the .skymodel image created by simobserve or simalma and used by one of those tasks to create an MS; if unspecified, will try to find one similar to your specified output image name)',
               'vis': 'Measurement Set(s) to image',
               'modelimage': 'lower resolution prior image to use in clean e.g. existing total power image',
               'imsize': 'output image size in pixels (x,y) or 0 to match model',
               'imdirection': 'set output image direction, (otherwise center on the model)',
               'cell': 'cell size with units e.g. "10arcsec" or "" to equal model',
               'interactive': 'interactive clean?  (make sure to set niter>0 also)',
               'niter': 'maximum number of iterations (0 for dirty image)',
               'threshold': 'flux level (+units) to stop cleaning',
               'weighting': 'weighting to apply to visibilities.  briggs will use robust=0.5',
               'mask': 'Cleanbox(es), mask image(s), region(s), or a level',
               'outertaper': 'uv-taper on outer baselines in uv-plane',
               'pbcor': 'correct the output of synthesis images for primary beam response?',
               'stokes': 'Stokes params to image',
               'featherimage': 'image (e.g. total power) to feather with new image',
               'analyze': '(only first 6 selected outputs will be displayed)',
               'showuv': 'display uv coverage',
               'showpsf': 'display synthesized (dirty) beam (ignored in single dish simulation)',
               'showmodel': 'display sky model at original resolution',
               'showconvolved': 'display sky model convolved with output clean beam',
               'showclean': 'display the synthesized image',
               'showresidual': 'display the clean residual image (ignored in single dish simulation)',
               'showdifference': 'display difference between output cleaned image and input model sky image convolved with output clean beam',
               'showfidelity': 'display fidelity (see help)',
               'graphics': 'display graphics at each stage to [screen|file|both|none]',
               'verbose': '',
               'overwrite': 'overwrite files starting with $project',
               'dryrun': 'only print information [experimental; only for interfermetric data]',
               'logfile': '',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['project']  = 'sim'
        a['image']  = True
        a['imagename']  = 'default'
        a['skymodel']  = ''
        a['vis']  = 'default'
        a['modelimage']  = ''
        a['imsize']  = [0, 0]
        a['imdirection']  = ''
        a['cell']  = ''
        a['interactive']  = False
        a['niter']  = 0
        a['threshold']  = '0.1mJy'
        a['weighting']  = 'natural'
        a['mask']  = []
        a['outertaper']  = ['']
        a['pbcor']  = True
        a['stokes']  = 'I'
        a['featherimage']  = ''
        a['analyze']  = False
        a['showuv']  = True
        a['showpsf']  = True
        a['showmodel']  = True
        a['showconvolved']  = False
        a['showclean']  = True
        a['showresidual']  = False
        a['showdifference']  = True
        a['showfidelity']  = True
        a['graphics']  = 'both'
        a['verbose']  = False
        a['overwrite']  = True
        a['dryrun']  = False
        a['logfile']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
simanalyze_pg = simanalyze_pg_()
