#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_simobserve import simobserve
from task_simobserve import casalog

class simobserve_pg_:
    __name__ = "simobserve"

    def __init__(self) :
       self.__bases__ = (simobserve_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, project=None,skymodel=None,inbright=None,indirection=None,incell=None,incenter=None,inwidth=None,complist=None,compwidth=None,setpointings=None,ptgfile=None,integration=None,direction=None,mapsize=None,maptype=None,pointingspacing=None,caldirection=None,calflux=None,obsmode=None,refdate=None,hourangle=None,totaltime=None,antennalist=None,sdantlist=None,sdant=None,thermalnoise=None,user_pwv=None,t_ground=None,t_sky=None,tau0=None,seed=None,leakage=None,graphics=None,verbose=None,overwrite=None,):

        """visibility simulation task
    -------------------------------
    project -- the root filename for all output files.
    -------------------------------
    skymodel -- input image (used as a model of the sky)
       * simalma requires a CASA or fits image. If you merely have a grid of 
         numbers, you will need to write them out as fits or write a 
         CASA script to read them in and use the ia tool to create an image 
         and insert the data.
       
       * simalma does NOT require a coordinate system in the header. If the
         coordinate information is incomplete, missing, or you would like to
         override it, set the appropriate "in" parameters. NOTE that setting 
         those parameters simply changes the header values, ignoring
         any values already in the image. No regridding is performed.
    
       * If you have a proper Coordinate System, simalma will do its best to
         generate visibilities from that, and then create a synthesis image
         according to the specified user parameters. 
    
       * You can manipulate an image header manually with the "imhead" task.
    
    inbright -- peak brightness in Jy/pixel, or "" for unchanged
       * NOTE: "unchanged" will take the numerical values in your image 
         and assume they are in Jy/pixel, even if it says some other unit 
         in the header. 
    indirection -- central direction, or "" for unchanged
    incell -- spatial pixel size, or "" for unchanged
    incenter -- frequency of center channel e.g. "89GHz", or "" for unchanged
    inwidth -- width of channels, or "" for unchanged - this should be a 
         string representing a quantity with units e.g. "10MHz"
       * NOTE: only works reliably with frequencies, not velocities
       * NOTE: it is not possible to change the number of spectral planes
         of the sky model, only to relabel them with different frequencies
         That kind of regridding can be accomplished with the CASA toolkit.
    -------------------------------
    complist -- component list model of the sky, added to or instead of skymodel
         see http://casaguides.nrao.edu/index.php?title=Simulation_Guide_Component_Lists_%28CASA_4.1%29
    compwidth -- bandwidth of components; if simulating from components only, 
         this defines the bandwidth of the MS and output images
    -------------------------------
    setpointings -- calculate a map of pointings, or if false, provide ptgfile
       * if graphics are on, display the pointings shown on the model image
    ptgfile -- a text file specifying directions in the following
         format, with optional integration times, e.g.
         #Epoch     RA          DEC      TIME(optional)
         J2000 23h59m28.10 -019d52m12.35 10.0
       * if the time column is not present in the file, it will use
         "integration" for all pointings.
       * NOTE: at this time the file should contain only science pointings:
         simobserve will observe these, then optionally the calibrator, 
         then the list of science pointings again, etc, until totaltime
         is used up. 
    integration --- Time interval for each integration e.g '10s'
       * NOTE: to simulate a "scan" longer than one integration, use 
         setpointings to generate a pointing file, and then edit the 
         file to increase the time at each point to be larger than 
         the parameter integration time.
    direction -- mosaic center direction e.g 'J2000 19h00m00 -40d00m00'
       * can optionally be a list of pointings
       * otherwise simobserve will pack mapsize according to maptype
    mapsize -- angular size of map 
       * set to "" to span the model image
    maptype -- hexagonal, square (rectangular raster), 
         "ALMA" for the same hex algorithm as the ALMA Cycle 1 OT 
         or "ALMA2012" for the algorithm used in the Cycle 0 OT
    pointingspacing -- spacing in between beams e.g '1arcsec'
         "0.25PB" to use 1/4 of the primary beam FWHM, 
         "nyquist" will use lambda/d/2,
         "" will use lambda/d/sqrt(3) for INT, lambda/d/3 for SD
    -------------------------------
    obsmode -- observation mode to calculate visibilities from a skymodel image
         (which may have been modified above), an optional component list, 
         and a pointing file (which also may have been generated above)
       * this parameter takes two possible values:
         - interferometer (or i)
         - singledish (or s)
       * if graphics are on, this observe step will display the array 
         (similar to plotants), the uv coverage, the synthesized (dirty) beam, 
         and ephemeris information
       * if simulating from component list, you should specify "compwidth", 
         the desired bandwidth.  There is not currently a way to specify the 
         spectrum of a component, so simulations from a componentlist only 
         will be continuum (1 chan)
    refdate -- date of simulated observation eg: '2014/05/21'
    hourangle -- hour angle of observation e.g. '-3h'
       * note that if you don't add a unit, it will assume hours
    totaltime --- total time of observation e.g '7200s' or if a number without
         units, interpreted as the number of times to repeat the map
    antennalist -- ascii file containing antenna positions.
         each row has x y z coordinates and antenna diameter; 
         header lines are required to specify the observatory name
         and coordinate system e.g. 
           # observatory=ALMA
           # coordsys=UTM
           # datum=WGS84
           # zone=19
       * standard arrays are found in your CASA data repository, 
         os.getenv("CASAPATH").split()[0]+"/data/alma/simmos/"        
       * if "", simobserve will not not produce an interferometric MS
       * a string of the form "alma;0.5arcsec" will be parsed into a full 
         12m ALMA configuration.  This only works for full ALMA and may fail 
         to find the standard configuration files on some systems 
         - see casaguides.nrao.edu for more information.
    caldirection -- an unresolved calibrator can be observed 
         interleaved with the science pointings.  
       * The calibrator is implemented as a point source clean component 
         with this specified direction and flux=calflux
    sdant -- the index of the antenna in the list to use for total 
         power.  defaults to the first antenna on the list. 
    -------------------------------
    thermalnoise -- add thermal noise 
       * this parameter takes two possible values:
       - tsys-atm: J. Pardo's ATM library will be used to construct an
           atmospheric profile for the ALMA site:
           altitude 5000m, ground pressure 650mbar, relhum=20%, 
           a water layer of user_pwv at altitude of 2km,
           the sky brightness temperature returned by ATM, 
           and internally tabulated receiver temperatures
       - tsys-manual: instead of using the ATM model, specify the zenith 
           sky brightness and opacity manually.  Noise is added and then
           the visibility flux scale is referenced above the atmosphere. 
       * In either mode, noise is calculated using an antenna spillover 
           efficiency of 0.96, taper of 0.86, 
           surface accuracy of 25 and 300 microns for ALMA and EVLA
           respectively (using the Ruze formula for surface efficiency),
           correlator efficiencies of 0.95 and 0.91 for ALMA and EVLA, 
           receiver temperatures for ALMA of 
               17, 30, 37, 51, 65, 83,147,196,175,230 K interpolated between
               35, 75,110,145,185,230,345,409,675,867 GHz,
           for EVLA of 
               500, 70,  60,  55,  100, 130, 350 K interpolated between
               0.33,1.47,4.89,8.44,22.5,33.5,43.3 GHz,
           for SMA of
               67,  116, 134, 500 K interpolated between
               212.,310.,383.,660. GHz
       * These are only approximate numbers and do not take into account 
         performance at edges of receiver bands, neither are they guaranteed
         to reflect the most recent measurements.  Caveat emptor and use the 
         sm tool to add noise if you want more precise control.
    t_ground -- ground/spillover temperature in K 
    user_pwv -- precipitable water vapor if constructing an atmospheric model
    t_sky -- atmospheric temperature in K [for tsys-manual]
    tau0 -- zenith opacity at observing frequency [for tsys-manual]
       * see casaguides.nrao.edu for more information on noise, 
         in particular how to add a phase screen using the toolkit
    seed -- random number seed for noise generation
    -------------------------------
    leakage -- add cross polarization corruption of this fractional magnitude
    
    graphics -- view plots on the screen, saved to file, both, or neither
    verbose -- print extra information to the logger and terminal
    overwrite -- overwrite existing files in the project subdirectory
    
    
    -------------------------------
    -------------------------------
    Output produced: (not all will always exist, depending on input parameters)
    To support different runs with different arrays, the names have the
    configuration name from antennalist appended.
    -------------------------------
    
    project.[cfg].skymodel = 4d input sky model image (optionally) scaled
    project.[cfg].skymodel.flat.regrid.conv = input sky regridded to match the
        output image, and convolved with the output clean beam
    project.[cfg].skymodel.png = diagnostic figure of sky model with pointings
    
    project.[cfg].ptg.txt = list of mosaic pointings
    project.[cfg].quick.psf = psf calculated from uv coverage
    project.[cfg].ms = noise-free measurement set
    project.[cfg].noisy.ms = corrupted measurement set 
    project.[cfg].observe.png = diagnostic figure of uv coverage and 
         visibilities
    	    
    project.[cfg].simobserve.last = saved input parameters for simobserve task

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'simobserve'
        myf['taskname'] = 'simobserve'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['project'] = project = myf['project']
            myparams['skymodel'] = skymodel = myf['skymodel']
            myparams['inbright'] = inbright = myf['inbright']
            myparams['indirection'] = indirection = myf['indirection']
            myparams['incell'] = incell = myf['incell']
            myparams['incenter'] = incenter = myf['incenter']
            myparams['inwidth'] = inwidth = myf['inwidth']
            myparams['complist'] = complist = myf['complist']
            myparams['compwidth'] = compwidth = myf['compwidth']
            myparams['setpointings'] = setpointings = myf['setpointings']
            myparams['ptgfile'] = ptgfile = myf['ptgfile']
            myparams['integration'] = integration = myf['integration']
            myparams['direction'] = direction = myf['direction']
            myparams['mapsize'] = mapsize = myf['mapsize']
            myparams['maptype'] = maptype = myf['maptype']
            myparams['pointingspacing'] = pointingspacing = myf['pointingspacing']
            myparams['caldirection'] = caldirection = myf['caldirection']
            myparams['calflux'] = calflux = myf['calflux']
            myparams['obsmode'] = obsmode = myf['obsmode']
            myparams['refdate'] = refdate = myf['refdate']
            myparams['hourangle'] = hourangle = myf['hourangle']
            myparams['totaltime'] = totaltime = myf['totaltime']
            myparams['antennalist'] = antennalist = myf['antennalist']
            myparams['sdantlist'] = sdantlist = myf['sdantlist']
            myparams['sdant'] = sdant = myf['sdant']
            myparams['thermalnoise'] = thermalnoise = myf['thermalnoise']
            myparams['user_pwv'] = user_pwv = myf['user_pwv']
            myparams['t_ground'] = t_ground = myf['t_ground']
            myparams['t_sky'] = t_sky = myf['t_sky']
            myparams['tau0'] = tau0 = myf['tau0']
            myparams['seed'] = seed = myf['seed']
            myparams['leakage'] = leakage = myf['leakage']
            myparams['graphics'] = graphics = myf['graphics']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(direction)==str: direction=[direction]
        if type(mapsize)==str: mapsize=[mapsize]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['project'] = project
        mytmp['skymodel'] = skymodel
        mytmp['inbright'] = inbright
        mytmp['indirection'] = indirection
        mytmp['incell'] = incell
        mytmp['incenter'] = incenter
        mytmp['inwidth'] = inwidth
        mytmp['complist'] = complist
        mytmp['compwidth'] = compwidth
        mytmp['setpointings'] = setpointings
        mytmp['ptgfile'] = ptgfile
        mytmp['integration'] = integration
        mytmp['direction'] = direction
        mytmp['mapsize'] = mapsize
        mytmp['maptype'] = maptype
        mytmp['pointingspacing'] = pointingspacing
        mytmp['caldirection'] = caldirection
        mytmp['calflux'] = calflux
        mytmp['obsmode'] = obsmode
        mytmp['refdate'] = refdate
        mytmp['hourangle'] = hourangle
        mytmp['totaltime'] = totaltime
        mytmp['antennalist'] = antennalist
        mytmp['sdantlist'] = sdantlist
        mytmp['sdant'] = sdant
        mytmp['thermalnoise'] = thermalnoise
        mytmp['user_pwv'] = user_pwv
        mytmp['t_ground'] = t_ground
        mytmp['t_sky'] = t_sky
        mytmp['tau0'] = tau0
        mytmp['seed'] = seed
        mytmp['leakage'] = leakage
        mytmp['graphics'] = graphics
        mytmp['verbose'] = verbose
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'simobserve.xml')

        casalog.origin('simobserve')
        if not trec.has_key('simobserve') or not casac.utils().verify(mytmp, trec['simobserve']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: simobserve           #####')
          casalog.post('')
          result = simobserve(project, skymodel, inbright, indirection, incell, incenter, inwidth, complist, compwidth, setpointings, ptgfile, integration, direction, mapsize, maptype, pointingspacing, caldirection, calflux, obsmode, refdate, hourangle, totaltime, antennalist, sdantlist, sdant, thermalnoise, user_pwv, t_ground, t_sky, tau0, seed, leakage, graphics, verbose, overwrite)
          casalog.post('')
          casalog.post('##### End Task: simobserve           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('simobserve', 'simobserve.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'simobserve'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('simobserve', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['project']  = 'sim'
        a['skymodel']  = ''
        a['complist']  = ''
        a['setpointings']  = True
        a['obsmode']  = 'int'
        a['thermalnoise']  = 'tsys-atm'
        a['leakage']  = 0.0
        a['graphics']  = 'both'
        a['verbose']  = False
        a['overwrite']  = True

        a['skymodel'] = {
                    0:odict([{'notvalue':''}, {'inbright':''}, {'indirection':''}, {'incell':''}, {'incenter':''}, {'inwidth':''}])}
        a['complist'] = {
                    0:odict([{'notvalue':''}, {'compwidth':'8GHz'}])}
        a['setpointings'] = {
                    0:odict([{'value':True}, {'integration':'10s'}, {'direction':''}, {'mapsize':['', '']}, {'maptype':'ALMA'}, {'pointingspacing':''}]), 
                    1:odict([{'value':False}, {'ptgfile':'$project.ptg.txt'}, {'integration':'10s'}])}
        a['obsmode'] = {
                    0:odict([{'value':'int'}, {'antennalist':'alma.out10.cfg'}, {'refdate':'2014/05/21'}, {'hourangle':'transit'}, {'totaltime':"7200s"}, {'caldirection':''}, {'calflux':'1Jy'}]), 
                    1:odict([{'value':'sd'}, {'sdantlist':'aca.tp.cfg'}, {'sdant':0}, {'refdate':'2014/05/21'}, {'hourangle':'transit'}, {'totaltime':"7200s"}]), 
                    2:odict([{'value':''}, {'antennalist':''}, {'sdantlist':''}, {'sdant':0}])}
        a['thermalnoise'] = {
                    0:odict([{'value':'tsys-atm'}, {'user_pwv':0.5}, {'t_ground':269.}, {'seed':11111}]), 
                    1:{'value':''}, 
                    2:{'value':'False'}, 
                    3:{'value':'F'}, 
                    4:odict([{'value':'tsys-manual'}, {'t_ground':269.}, {'t_sky':263.}, {'tau0':0.1}, {'seed':11111}]), 
                    5:{'value':''}, 
                    6:{'value':'False'}, 
                    7:{'value':'F'}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='simobserve', subkey=None):
        desc={'simobserve': 'visibility simulation task',
               'project': 'root prefix for output file names',
               'skymodel': 'model image to observe',
               'inbright': 'scale surface brightness of brightest pixel e.g. "1.2Jy/pixel"',
               'indirection': 'set new direction e.g. "J2000 19h00m00 -40d00m00"',
               'incell': 'set new cell/pixel size e.g. "0.1arcsec"',
               'incenter': 'set new frequency of center channel e.g. "89GHz" (required even for 2D model)',
               'inwidth': 'set new channel width e.g. "10MHz" (required even for 2D model)',
               'complist': 'componentlist to observe',
               'compwidth': 'bandwidth of components',
               'setpointings': '',
               'ptgfile': 'list of pointing positions',
               'integration': 'integration (sampling) time',
               'direction': '"J2000 19h00m00 -40d00m00" or "" to center on model',
               'mapsize': 'angular size of map or "" to cover model',
               'maptype': 'hexagonal, square (raster), ALMA, etc',
               'pointingspacing': 'spacing in between pointings or "0.25PB" or "" for ALMA default INT=lambda/D/sqrt(3), SD=lambda/D/3 ',
               'caldirection': 'pt source calibrator [experimental]',
               'calflux': '',
               'obsmode': 'observation mode to simulate [int(interferometer)|sd(singledish)|""(none)]',
               'refdate': 'date of observation - not critical unless concatting simulations',
               'hourangle': 'hour angle of observation center e.g. "-3:00:00", "5h", "-4.5" (a number without units will be interpreted as hours), or "transit" ',
               'totaltime': 'total time of observation or number of repetitions',
               'antennalist': 'interferometer antenna position file',
               'sdantlist': 'single dish antenna position file',
               'sdant': 'single dish antenna index in file',
               'thermalnoise': 'add thermal noise: [tsys-atm|tsys-manual|""]',
               'user_pwv': 'Precipitable Water Vapor in mm',
               't_ground': 'ambient temperature',
               't_sky': 'atmospheric temperature',
               'tau0': 'zenith opacity',
               'seed': 'random number seed',
               'leakage': 'cross polarization (interferometer only)',
               'graphics': 'display graphics at each stage to [screen|file|both|none]',
               'verbose': '',
               'overwrite': 'overwrite files starting with $project',

              }

#
# Set subfields defaults if needed
#
        if(subkey == 'True'):
          desc['direction'] = 'center of map or "" to center on the model'
        if(subkey == 'False'):
          desc['integration'] = 'integration time (see below)'
        if(subkey == ''):
          desc['antennalist'] = 'antenna info can be used to calculate the primary beam'
        if(subkey == ''):
          desc['sdantlist'] = 'antenna info can be used to calculate the primary beam'

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['project']  = 'sim'
        a['skymodel']  = ''
        a['inbright']  = ''
        a['indirection']  = ''
        a['incell']  = ''
        a['incenter']  = ''
        a['inwidth']  = ''
        a['complist']  = ''
        a['compwidth']  = '"8GHz"'
        a['setpointings']  = True
        a['ptgfile']  = '$project.ptg.txt'
        a['integration']  = '10s'
        a['direction']  = ['']
        a['mapsize']  = ['', '']
        a['maptype']  = 'hexagonal'
        a['pointingspacing']  = ''
        a['caldirection']  = ''
        a['calflux']  = '1Jy'
        a['obsmode']  = 'int'
        a['refdate']  = '2014/01/01'
        a['hourangle']  = 'transit'
        a['totaltime']  = '7200s'
        a['antennalist']  = ''
        a['sdantlist']  = 'aca.tp.cfg'
        a['sdant']  = 0
        a['thermalnoise']  = 'tsys-atm'
        a['user_pwv']  = 0.5
        a['t_ground']  = 270.
        a['t_sky']  = 260.
        a['tau0']  = 0.1
        a['seed']  = 11111
        a['leakage']  = 0.0
        a['graphics']  = 'both'
        a['verbose']  = False
        a['overwrite']  = True

        if a.has_key(paramname) :
	      return a[paramname]
simobserve_pg = simobserve_pg_()
