#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_split import split
from task_split import casalog

class split_pg_:
    __name__ = "split"

    def __init__(self) :
       self.__bases__ = (split_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,outputvis=None,datacolumn=None,field=None,spw=None,width=None,antenna=None,timebin=None,timerange=None,scan=None,intent=None,array=None,uvrange=None,correlation=None,observation=None,combine=None,keepflags=None,keepmms=None,):

        """Create a visibility subset from an existing visibility set

    Split is the general purpose program to make a new data set that is a
    subset or averaged form of an existing data set.  General selection
    parameters are included, and one or all of the various data columns
    (DATA, LAG_DATA and/or FLOAT_DATA, and possibly MODEL_DATA and/or
    CORRECTED_DATA) can be selected.
    
    Split is often used after the initial calibration of the data to make a
    smaller measurement set with only the data that will be used in
    further flagging, imaging and/or self-calibration.  split can
    average over frequency (channels) and time (integrations).

    With the keepmms parameter, split can be run parallelized on multi-MS
    input.

        Keyword arguments:
        vis -- Name of input visibility file
                default: none; example: vis='ngc5921.ms'
        outputvis -- Name of output visibility file
                default: none; example: outputvis='ngc5921_src.ms'

        datacolumn -- Which data column to split out
                default='corrected'; example: datacolumn='data'
                Options: 'data', 'model', 'corrected', 'all',
                'float_data', 'lag_data', 'float_data,data', and
                'lag_data,data'.
                N.B.: 'all' = whichever of the above that are present.
                Otherwise the selected column will go to DATA (or
                FLOAT_DATA) in the output.
                Splitting with the default datacolumn='corrected'
                before clean is normally required for self-calibration!

        --- Data Selection (see help par.selectdata for more detailed
            information)

        field -- Select field using field id(s) or field name(s).
                  [run listobs to obtain the list id's or names]
               default: ''=all fields If field string is a non-negative
               integer, it is assumed to be a field index
               otherwise, it is assumed to be a field name
               field='0~2'; field ids 0,1,2
               field='0,4,5~7'; field ids 0,4,5,6,7
               field='3C286,3C295'; fields named 3C286 and 3C295
               field = '3,4C*'; field id 3, all names starting with 4C
        spw -- Select spectral window/channels
               default: ''=all spectral windows and channels
               spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
               spw='<2';  spectral windows less than 2 (i.e. 0,1)
               spw='0:5~61'; spw 0, channels 5 to 61
               spw='0,10,3:3~45'; spw 0,10 all channels, spw 3 - chans 3 to 45.
               spw='0~2:2~6'; spw 0,1,2 with channels 2 through 6 in each.
               spw = '*:3~64'  channels 3 through 64 for all sp id's
                       spw = ' :3~64' will NOT work.
               split does not support multiple channel ranges per spectral
               window (';') because it is not clear whether to keep the ranges
               in the original spectral window or make a new spectral window
               for each additional range.
	width -- Defines the number of channel to average to form the one
	         output channel.
		 default: '1' => no channel averaging
		 example: width=[2,3] => average 2 channels of 1st
		          spectral window selected and 3 in the second one.
        antenna -- Select data based on antenna/baseline
               default: '' (all)
                Non-negative integers are assumed to be antenna indices, and
                anything else is taken as an antenna name.

                Examples:
                antenna='5&6': baseline between antenna index 5 and index 6.
                antenna='VA05&VA06': baseline between VLA antenna 5 and 6.
                antenna='5&6;7&8': baselines 5-6 and 7-8
                antenna='5': all baselines with antenna 5
                antenna='5,6,10': all baselines including antennas 5, 6, or 10
                antenna='5,6,10&': all baselines with *only* antennas 5, 6, or
                                       10.  (cross-correlations only.  Use &&
                                       to include autocorrelations, and &&&
                                       to get only autocorrelations.)
                antenna='!ea03,ea12,ea17': all baselines except those that
                                           include EVLA antennas ea03, ea12, or
                                           ea17.
        timebin -- Interval width for time averaging.
                   default: '0s' or '-1s' (no averaging)
                   example: timebin='30s'
                            '10' means '10s'
        timerange -- Select data based on time range:
               default = '' (all); examples,
               timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
               Note: if YYYY/MM/DD is missing date, timerange defaults to the
               first day in the dataset
               timerange='09:14:0~09:54:0' picks 40 min on first day
               timerange='25:00:00~27:30:00' picks 1 hr to 3 hr 30min
               on next day
               timerange='09:44:00' data within one integration of time
               timerange='>10:24:00' data after this time
        array -- (Sub)array number range
            default: ''=all
	uvrange -- Select data within uvrange (default units meters)
            default: ''=all; example:
            uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lambda
            uvrange='>4klambda';uvranges greater than 4 kilo-lambda
            uvrange='0~1000km'; uvrange in kilometers
	scan -- Scan number range
            default: ''=all
        intent -- Select by scan intent (state).  Case sensitive.
            default: '' = all
            Examples:
            intent = 'CALIBRATE_ATMOSPHERE_REFERENCE'
            intent = 'calibrate_atmosphere_reference'.upper() # same as above
            # Select states that include one or both of CALIBRATE_WVR.REFERENCE
            # or OBSERVE_TARGET_ON_SOURCE.
            intent = 'CALIBRATE_WVR.REFERENCE, OBSERVE_TARGET_ON_SOURCE'
        correlation -- Select correlations, e.g. 'rr, ll' or ['XY', 'YX'].
                       default '' (all).
        observation -- Select by observation ID(s).
                       default: '' = all
        combine -- Let time bins span changes in scan and/or state.
                   default = '' (separate time bins by both of the above)
                   combine = 'scan': Can be useful when the scan number
                                     goes up with each integration,
                                     as in many WSRT MSes.
                   combine = ['scan', 'state']: disregard scan and state
                                                numbers when time averaging.
                   combine = 'state,scan': Same as above.
        keepflags -- If practical, keep completely flagged rows instead of
                     dropping them.  This has absolutely no effect on averaging
                     calculations, or partially flagged rows.  All of the
                     channels and correlations of a row must be flagged for it
                     to be droppable, and a row must be well defined to be
                     keepable.  The latter condition means that this option has
                     no effect on time averaging - in that case fully flagged
                     rows are automatically omitted.  Regardless of this
                     parameter, flagged data is never included in averaging
                     calculations.

                     The only time keepflags matters is if 
                     1. the input MS has some completely flagged rows
                     and
                     2. time averaging is not being done.

                     Then, if keepflags is False, the completely flagged rows
                     will be omitted from the output MS.  Otherwise, they will
                     be included (subject to the selection parameters).

        keepmms -- If true and the input is a multi-MS, make the output one, too.
	           Otherwise, the output will be a normal MS without partitioning. 
		   (experimental)
                   Default: False


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'split'
        myf['taskname'] = 'split'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['outputvis'] = outputvis = myf['outputvis']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['width'] = width = myf['width']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['timebin'] = timebin = myf['timebin']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['scan'] = scan = myf['scan']
            myparams['intent'] = intent = myf['intent']
            myparams['array'] = array = myf['array']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['correlation'] = correlation = myf['correlation']
            myparams['observation'] = observation = myf['observation']
            myparams['combine'] = combine = myf['combine']
            myparams['keepflags'] = keepflags = myf['keepflags']
            myparams['keepmms'] = keepmms = myf['keepmms']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['outputvis'] = outputvis
        mytmp['datacolumn'] = datacolumn
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['width'] = width
        mytmp['antenna'] = antenna
        mytmp['timebin'] = timebin
        mytmp['timerange'] = timerange
        mytmp['scan'] = scan
        mytmp['intent'] = intent
        mytmp['array'] = array
        mytmp['uvrange'] = uvrange
        mytmp['correlation'] = correlation
        mytmp['observation'] = observation
        mytmp['combine'] = combine
        mytmp['keepflags'] = keepflags
        mytmp['keepmms'] = keepmms
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'split.xml')

        casalog.origin('split')
        if not trec.has_key('split') or not casac.utils().verify(mytmp, trec['split']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: split           #####')
          casalog.post('')
          result = split(vis, outputvis, datacolumn, field, spw, width, antenna, timebin, timerange, scan, intent, array, uvrange, correlation, observation, combine, keepflags, keepmms)
          casalog.post('')
          casalog.post('##### End Task: split           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('split', 'split.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'split'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('split', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['outputvis']  = ''
        a['datacolumn']  = 'corrected'
        a['field']  = ''
        a['spw']  = ''
        a['width']  = 1
        a['antenna']  = ''
        a['timebin']  = '0s'
        a['timerange']  = ''
        a['scan']  = ''
        a['intent']  = ''
        a['array']  = ''
        a['uvrange']  = ''
        a['correlation']  = ''
        a['observation']  = ''
        a['keepflags']  = True
        a['keepmms']  = False

        a['timebin'] = {
                    0:odict([{'notvalue':'0s'}, {'combine':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='split', subkey=None):
        desc={'split': 'Create a visibility subset from an existing visibility set',
               'vis': 'Name of input measurement set',
               'outputvis': 'Name of output measurement set',
               'datacolumn': 'Which data column(s) to split out',
               'field': 'Select field using ID(s) or name(s)',
               'spw': 'Select spectral window/channels',
               'width': 'Number of channels to average to form one output channel',
               'antenna': 'Select data based on antenna/baseline',
               'timebin': 'Bin width for time averaging',
               'timerange': 'Select data by time range',
               'scan': 'Select data by scan numbers',
               'intent': 'Select data by scan intents',
               'array': 'Select (sub)array(s) by array ID number',
               'uvrange': 'Select data by baseline length',
               'correlation': 'Select correlations',
               'observation': 'Select by observation ID(s)',
               'combine': 'Let time bins span changes in scan and/or state',
               'keepflags': 'If practical, keep *completely flagged rows* instead of dropping them.',
               'keepmms': 'If the input is a multi-MS, make the output one, too. (experimental)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['outputvis']  = ''
        a['datacolumn']  = 'corrected'
        a['field']  = ''
        a['spw']  = ''
        a['width']  = 1
        a['antenna']  = ''
        a['timebin']  = '0s'
        a['timerange']  = ''
        a['scan']  = ''
        a['intent']  = ''
        a['array']  = ''
        a['uvrange']  = ''
        a['correlation']  = ''
        a['observation']  = ''
        a['combine']  = ''
        a['keepflags']  = True
        a['keepmms']  = False

        if a.has_key(paramname) :
	      return a[paramname]
split_pg = split_pg_()
