#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_tsdbaseline import tsdbaseline
from task_tsdbaseline import casalog

class tsdbaseline_pg_:
    __name__ = "tsdbaseline"

    def __init__(self) :
       self.__bases__ = (tsdbaseline_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,datacolumn=None,antenna=None,field=None,spw=None,timerange=None,scan=None,pol=None,maskmode=None,thresh=None,avg_limit=None,edge=None,blmode=None,dosubtract=None,blformat=None,bloutput=None,bltable=None,blfunc=None,order=None,npiece=None,applyfft=None,fftmethod=None,fftthresh=None,addwn=None,rejwn=None,clipthresh=None,clipniter=None,blparam=None,verify=None,verbose=None,showprogress=None,minnrow=None,outfile=None,overwrite=None,):

        """Fit/subtract a spectral baseline 
-----------------
Keyword arguments
-----------------
infile -- name of input SD dataset
datacolumn -- name of data column to be used
        options: 'data', 'float_data', or 'corrected_data'
        default: 'data'
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)/channels
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
                 spw='0:5~61' (IF ID 0; channels 5 to 61; all channels)
                 spw='3:10~20;50~60' (select multiple channel ranges within IF ID 3)
                 spw='3:10~20,4:0~30' (select different channel ranges for IF IDs 3 and 4)
                 spw='1~4;6:15~48' (for channels 15 through 48 for IF IDs 1,2,3,4 and 6)
        this selection is in addition to the other selections to data
timerange -- select data by time range
        default: '' (use all)
        example: timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                 Note: YYYY/MM/DD can be dropped as needed:
                 timerange='09:14:00~09:54:00' # this time range
                 timerange='09:44:00' # data within one integration of time
                 timerange='>10:24:00' # data after this time
                 timerange='09:44:00+00:13:00' #data 13 minutes after time
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
maskmode -- mode of setting additional channel masks. When blmode='apply'
             and/or blfunc='variable', maskmode and its subparameters
             are ignored.
        options: 'list' ('auto' and 'interact' will be available later)
        default: 'list'
        example: maskmode='auto' runs linefinder to detect line regions 
                 to be excluded from fitting. this mode requires three 
                 expandable parameters: thresh, avg_limit, and edge.
                 USE WITH CARE! May need to tweak the expandable parameters.
                 maskmode='list' uses the given masklist only: no additional 
                 masks applied.
                 maskmode='interact' allows users to manually modify the 
                 mask regions by dragging mouse on the spectrum plotter GUI.
                 use LEFT or RIGHT button to add or delete regions, 
                 respectively.
    >>> maskmode expandable parameters
        thresh -- S/N threshold for linefinder. a single channel S/N ratio
                  above which the channel is considered to be a detection. 
                default: 5
        avg_limit -- channel averaging for broad lines. a number of
                     consecutive channels not greater than this parameter
                     can be averaged to search for broad lines.
                default: 4
        edge -- channels to drop at beginning and end of spectrum
                default: 0
                example: edge=[1000] drops 1000 channels at beginning AND end.
                         edge=[1000,500] drops 1000 from beginning and 500
                         from end.
        Note: For bad baselines threshold should be increased,
        and avg_limit decreased (or even switched off completely by
        setting this parameter to 1) to avoid detecting baseline
        undulations instead of real lines.
blmode -- baselining mode. 
        options: 'fit', 'apply'
        default: 'fit'
        example: blmode='fit' calculates the best-fit baseline based on 
                 given baseline type, then (if you set dosubtract=True) 
                 subtract it from each spectrum. The information about 
                 best-fit baselines (baseline type, order, coefficients, 
                 etc.) can be stored in various formats (cf. blformat).
                 blmode='apply' reads a baseline table as well as input 
                 MS, reproduces the best-fit baseline via info written 
                 in the baseline table, then subtracts it from each 
                 spectrum. 
    >>> blmode expandable parameters
        dosubtract -- execute baseline subtraction in addition to fitting.
                      Note that dosubtract=False will be ignored if 
                      bloutput is given, that is, baseline subtraction 
                      will be always executed for the input MS in case 
                      bloutput is not specified.
                options: (bool) True, False
                default: True
        blformat -- format(s) of file(s) in which best-fit parameters are 
                    written. 
                options: 'text', 'csv', 'table', '', and a list of these 
                         available formats except for ''.
                default: 'text'
                example: blformat='text' outputs an ascii text file with 
                         the best-fit baseline parameters written in 
                         human-readable format. may be good to read, but 
                         you should mind it might be huge.
                         blformat='csv' outputs a CSV file.
                         blformat='table' outputs a baseline table which 
                         can be used to apply afterwards.
                         blformat='' doesn't output any parameter file.
                         blformat=['csv','table'] outputs both a CSV 
                         file and a baseline table.
        bloutput -- name(s) of file(s) in which best-fit parameters are 
                    written. If bloutput is '', name(s) of baseline 
                    parameter file(s) will be set as follows: 
                    <outfile>_blparam.txt for blformat='text', 
                    <outfile>_blparam.csv for blformat='csv', and 
                    <outfile>_bltable.
                    In case bloutput is not '', blformat and bloutput 
                    must have the same length.
                default: ''
        bltable -- name of baseline table to apply
                default: ''
blfunc -- baseline model function. In cases blmode='apply' or blparam is 
          set, blfunc and its subparameters are ignored.
        options: 'poly', 'chebyshev', 'cspline' or 'variable'
                 ('sinusoid' will be available later)
        default: 'poly'
        example: blfunc='poly' uses a single polynomial line of 
                 any order which should be given as an expandable 
                 parameter 'order' to fit baseline. 
                 blfunc='chebyshev' uses Chebyshev polynomials. 
                 blfunc='cspline' uses a cubic spline function, a piecewise 
                 cubic polynomial having C2-continuity (i.e., the second 
                 derivative is continuous at the joining points). 
                 blfunc='sinusoid' uses a combination of sinusoidal curves. 
        NOTE blfunc='variable' IS EXPERT MODE!!!
    >>> blfunc expandable parameters
        order -- order of baseline model function
                options: (int) (<0 turns off baseline fitting)
                default: 5
                example: typically in range 2-9 (higher values
                         seem to be needed for GBT)
        npiece -- number of the element polynomials of cubic spline curve
                options: (int) (<0 turns off baseline fitting)
                default: 2
        applyfft -- automatically set wave numbers of sinusoidal functions 
                    for fitting by applying some method like FFT.
                options: (bool) True, False
                default: True
        fftmethod -- method to be used when applyfft=True. Now only 
                     'fft' is available and it is the default.
        fftthresh -- threshold to select wave numbers to be used for 
                     sinusoidal fitting. both (float) and (str) accepted.
                     given a float value, the unit is set to sigma.
                     for string values, allowed formats include:
                     'xsigma' or 'x' (= x-sigma level. e.g., '3sigma'), or
                     'topx' (= the x strongest ones, e.g. 'top5').
                default is 3.0 (unit: sigma).
        addwn -- additional wave number(s) of sinusoids to be used 
                 for fitting. 
                 (list) and (int) are accepted to specify every
                 wave numbers. also (str) can be used in case
                 you need to specify wave numbers in a certain range.
                 default: [0] (i.e., constant is subtracted at least)
                 example: 0
                          [0,1,2]
                          'a-b' (= a, a+1, a+2, ..., b-1, b),
                          '<a'  (= 0,1,...,a-2,a-1),
                          '>=a' (= a, a+1, ... up to the maximum wave
                                   number corresponding to the Nyquist
                                   frequency for the case of FFT).
        rejwn -- wave number(s) of sinusoid NOT to be used for fitting.
                 can be set just as addwn but has higher priority:
                 wave numbers which are specified both in addwn
                 and rejwn will NOT be used. 
                 default: []
        clipthresh -- clipping threshold for iterative fitting
                 default: 3
        clipniter -- maximum iteration number for iterative fitting
                 default: 0 (no iteration, i.e., no clipping)
        blparam -- the name of text file that stores per spectrum fit
                   parameters. See below for details of format.
verify -- interactively verify the results of operation for each spectrum.
          When verify = True, for each input spectrum, spectra
          before and after the operation are displayed in a plot
          window. At the prompt there are four choices of action:
          'Y' (accept the operation and continue to the next input
          spectrum), 'N' (reject the operation and continue to the 
          next input spectrum), 'A' (accept the current operation
          and continue non-interactively), and 'R' (reject the
          current operation and exit from operation).
          Note that when the operation is rejected by 'N' or 'R',
          no operation is done to the spectrum/spectra.
        options: (bool) True,False
        default: False
        NOTE: Currently available only when blfunc='poly'
verbose -- output fitting results to logger. if False, the fitting results
           including coefficients, residual rms, etc., are not output to
           the CASA logger, while the processing speed gets faster.
        options: (bool) True, False
        default: True
showprogress -- show progress status for large data
        options: (bool) True, False
        default: True
    >>> showprogress expandable parameter
        minnrow -- minimum number of input spectra to show progress status
                 default: 1000
outfile -- name of output file
        default: '' (<infile>_bs)
overwrite -- overwrite the output file if already exists
        options: (bool) True, False
        default: False
        NOTE this parameter is ignored when outform='ASCII'


-----------
DESCRIPTION
-----------

Task tsdbaseline performs baseline fitting/subtraction for single-dish spectra.
The fit parameters, terms and rms of baseline can be saved into an ascii file 
or baseline table. Subtracting baseline from data in input MS using existing 
baseline table is also possible.

-----------------------
BASELINE MODEL FUNCTION
-----------------------
The list of available model functions are shown above (see Keyword arguments
section). In general 'cspline' or 'chebyshev' are recommended since they are
more stable than others. 'poly' will work for lower order but will be unstable
for higher order fitting. 'sinusoid' is kind of special mode that will be
useful for the data that clearly shows standing wave in the spectral baseline.

----------------------------------
SIGMA CLIPPING (ITERATIVE FITTING)
----------------------------------
In general least square fitting is strongly affected by an extreme data
so that the resulting fit makes worse. Sigma clipping is an iterative
baseline fitting with data clipping based on a certain threshold. Threshold
is set as a certain factor times rms of the resulting (baseline subtracted)
spectra. If sigma clipping is on, baseline fit/removal is performed several
times. After each baseline subtraction, the data whose absolute value is
above threshold are detected and those data are excluded from the next round
of fitting. By using sigma clipping, extreme data are excluded from the
fit so that resulting fit is more robust.

The user is able to control a multiplication factor using parameter
clipthresh for clipping threshold based on rms. Actual threshold for sigma
clipping will be (clipthresh) x (rms of spectra). Also, the user can specify
number of maximum iteration to the parameter clipniter.

In general, sigma clipping will lower the performance since it increases
number of fits per spectra. However, it is strongly recommended to turn
on sigma clipping unless you are sure that the data is free from any kind
of extreme values that may affect the fit.


----------------------------------
PER SPECTRUM FIT PARAMETERS
----------------------------------
Per spectrum baseline fitting parameter is accepted in blfunc='variable'.
Note this is an expert mode. The fitting parameters should be defined in
a text file for each spectrum in the input MS. The text file should store
commpa separated values in order of:
row ID, polarization ID, masklist, blfunc, order, npiece, nwave,clipthresh,
clipniter, use_linefinder, thresh, edge, chan_avg_limit.
Each row in the text file must contain the following keys and values:
* 'row': row number,
* 'blfunc': function name.
     available ones include, 'poly', 'chebyshev', 'cspline' and 'sinusoid', 
* 'order': maximum order of polynomial. needed when blfunc='poly'
     or 'chebyshev', 
* 'npiece': number or piecewise polynomial. needed when blfunc='cspline',
and
* 'nwave': a list of sinusoidal wave numbers. needed when blfunc='sinusoid'.

example:
#row,pol,masklist,blfunc,order,npiece,nwave,clipthresh,clipniter,use_lf,thres,edge,avg_limit
0,0,[[500,1600]],poly,5,,,,,,,,
0,1,,chebyshev,10,,,,,,,,
1,0,,cspline,,4,,,,,,,
1,1,,sinusoid,,,[0,1,2,3],,,,,,

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'tsdbaseline'
        myf['taskname'] = 'tsdbaseline'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['scan'] = scan = myf['scan']
            myparams['pol'] = pol = myf['pol']
            myparams['maskmode'] = maskmode = myf['maskmode']
            myparams['thresh'] = thresh = myf['thresh']
            myparams['avg_limit'] = avg_limit = myf['avg_limit']
            myparams['edge'] = edge = myf['edge']
            myparams['blmode'] = blmode = myf['blmode']
            myparams['dosubtract'] = dosubtract = myf['dosubtract']
            myparams['blformat'] = blformat = myf['blformat']
            myparams['bloutput'] = bloutput = myf['bloutput']
            myparams['bltable'] = bltable = myf['bltable']
            myparams['blfunc'] = blfunc = myf['blfunc']
            myparams['order'] = order = myf['order']
            myparams['npiece'] = npiece = myf['npiece']
            myparams['applyfft'] = applyfft = myf['applyfft']
            myparams['fftmethod'] = fftmethod = myf['fftmethod']
            myparams['fftthresh'] = fftthresh = myf['fftthresh']
            myparams['addwn'] = addwn = myf['addwn']
            myparams['rejwn'] = rejwn = myf['rejwn']
            myparams['clipthresh'] = clipthresh = myf['clipthresh']
            myparams['clipniter'] = clipniter = myf['clipniter']
            myparams['blparam'] = blparam = myf['blparam']
            myparams['verify'] = verify = myf['verify']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['showprogress'] = showprogress = myf['showprogress']
            myparams['minnrow'] = minnrow = myf['minnrow']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(edge)==int: edge=[edge]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['datacolumn'] = datacolumn
        mytmp['antenna'] = antenna
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['timerange'] = timerange
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['maskmode'] = maskmode
        mytmp['thresh'] = thresh
        mytmp['avg_limit'] = avg_limit
        mytmp['edge'] = edge
        mytmp['blmode'] = blmode
        mytmp['dosubtract'] = dosubtract
        mytmp['blformat'] = blformat
        mytmp['bloutput'] = bloutput
        mytmp['bltable'] = bltable
        mytmp['blfunc'] = blfunc
        mytmp['order'] = order
        mytmp['npiece'] = npiece
        mytmp['applyfft'] = applyfft
        mytmp['fftmethod'] = fftmethod
        mytmp['fftthresh'] = fftthresh
        mytmp['addwn'] = addwn
        mytmp['rejwn'] = rejwn
        mytmp['clipthresh'] = clipthresh
        mytmp['clipniter'] = clipniter
        mytmp['blparam'] = blparam
        mytmp['verify'] = verify
        mytmp['verbose'] = verbose
        mytmp['showprogress'] = showprogress
        mytmp['minnrow'] = minnrow
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'tsdbaseline.xml')

        casalog.origin('tsdbaseline')
        if not trec.has_key('tsdbaseline') or not casac.utils().verify(mytmp, trec['tsdbaseline']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: tsdbaseline           #####')
          casalog.post('')
          result = tsdbaseline(infile, datacolumn, antenna, field, spw, timerange, scan, pol, maskmode, thresh, avg_limit, edge, blmode, dosubtract, blformat, bloutput, bltable, blfunc, order, npiece, applyfft, fftmethod, fftthresh, addwn, rejwn, clipthresh, clipniter, blparam, verify, verbose, showprogress, minnrow, outfile, overwrite)
          casalog.post('')
          casalog.post('##### End Task: tsdbaseline           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('tsdbaseline', 'tsdbaseline.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'tsdbaseline'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('tsdbaseline', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['datacolumn']  = 'data'
        a['antenna']  = 0
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['maskmode']  = 'list'
        a['blmode']  = 'fit'
        a['blfunc']  = 'poly'
        a['verify']  = False
        a['verbose']  = True
        a['showprogress']  = True
        a['outfile']  = ''
        a['overwrite']  = False

        a['maskmode'] = {
                    0:{'value':'list'}, 
                    1:odict([{'value':'auto'}, {'thresh':5.0}, {'avg_limit':4}, {'edge':[0]}]), 
                    2:{'value':'interact'}}
        a['blmode'] = {
                    0:odict([{'value':'fit'}, {'dosubtract':True}, {'blformat':'table'}, {'bloutput':''}]), 
                    1:odict([{'value':'apply'}, {'bltable':''}])}
        a['blfunc'] = {
                    0:odict([{'value':'poly'}, {'order':5}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    1:odict([{'value':'chebyshev'}, {'order':5}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    2:odict([{'value':'cspline'}, {'npiece':2}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    3:odict([{'value':'variable'}, {'blparam':''}])}
        a['showprogress'] = {
                    0:odict([{'value':True}, {'minnrow':1000}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='tsdbaseline', subkey=None):
        desc={'tsdbaseline': 'Fit/subtract a spectral baseline ',
               'infile': 'name of input SD dataset',
               'datacolumn': 'name of data column to be used [\'data\', \'float_data\', or \'corrected_data\']',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\'',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'timerange': 'select data by time range, e.g. \'09:14:0~09:54:0\' (\'\'=all) (see examples in help)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g. \'0,1\' (\'\'=all)',
               'maskmode': 'mode of setting additional channel masks',
               'thresh': 'S/N threshold for linefinder',
               'avg_limit': 'channel averaging for broad lines',
               'edge': 'channels to drop at beginning and end of spectrum',
               'blmode': 'baselining mode [\'fit\' or \'apply\']',
               'dosubtract': 'subtract baseline from input data [True, False] ',
               'blformat': 'format(s) of file(s) in which best-fit parameters are written',
               'bloutput': 'name(s) of file(s) in which best-fit parameters are written',
               'bltable': 'name of baseline table to apply',
               'blfunc': 'baseline model function',
               'order': 'order of baseline model function',
               'npiece': 'number of element polynomials for cubic spline curve',
               'applyfft': 'automatically set wave numbers of sinusoids',
               'fftmethod': 'method for automatically set wave numbers of sinusoids',
               'fftthresh': 'threshold to select wave numbers of sinusoids',
               'addwn': 'additional wave numbers to use',
               'rejwn': 'wave numbers NOT to use',
               'clipthresh': 'clipping threshold for iterative fitting',
               'clipniter': 'maximum iteration number for iterative fitting',
               'blparam': 'text file that stores per spectrum fit parameters',
               'verify': 'interactively verify the results of operation for each spectrum (see description in help)',
               'verbose': 'output fitting results to logger',
               'showprogress': 'show progress status for large data',
               'minnrow': 'minimum number of input spectra to show progress status',
               'outfile': 'name of output file',
               'overwrite': 'overwrite the output file if already exists',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['datacolumn']  = 'data'
        a['antenna']  = 0
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['maskmode']  = 'list'
        a['thresh']  = 5.0
        a['avg_limit']  = 4
        a['edge']  = [0]
        a['blmode']  = 'fit'
        a['dosubtract']  = True
        a['blformat']  = 'table'
        a['bloutput']  = ''
        a['bltable']  = ''
        a['blfunc']  = 'poly'
        a['order']  = 5
        a['npiece']  = 2
        a['applyfft']  = True
        a['fftmethod']  = 'fft'
        a['fftthresh']  = 3.0
        a['addwn']  = [0]
        a['rejwn']  = []
        a['clipthresh']  = 3.0
        a['clipniter']  = 0
        a['blparam']  = ''
        a['verify']  = False
        a['verbose']  = True
        a['showprogress']  = True
        a['minnrow']  = 1000
        a['outfile']  = ''
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
tsdbaseline_pg = tsdbaseline_pg_()
