#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_visstat import visstat
from task_visstat import casalog

class visstat_pg_:
    __name__ = "visstat"

    def __init__(self) :
       self.__bases__ = (visstat_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,axis=None,datacolumn=None,useflags=None,spw=None,field=None,selectdata=None,antenna=None,uvrange=None,timerange=None,correlation=None,scan=None,array=None,observation=None,):

        """Displays statistical information from a Measurement Set, or from a Multi-MS

        This task returns statistical information about data in a Measurement
        Set or Multi-MS.

        The following values are computed: mean value, sum of values,
        sum of squared values, median, median absolute deviation, quartile,
        minimum, maximum, variance, standard deviation, and root mean square.

        The following axes are supported: uvw, flag, weight, sigma, antenna1,
        antenna2, feed1, feed2, field_id, array_id, data_desc_id, flag_row,
        interval, scan, scan_number, time, weight_spectrum, amp, amplitude,
        phase, real, imag, imaginary, and uvrange.

        Optionally, the statistical information can be computed based only
        on a given subset of the measurement set.

        Note: If the MS consists of inhomogeneous data, for example several
        spectral windows each having a different number of channels, it may be
        necessary to use selection parameters to select a homogeneous subset of
        the MS, e.g. spw='2'.

        Keyword arguments:

        vis  --- Name of input Measurement Set or Multi-MS
                  default: '', example: vis='my.ms'

        axis -- Which data to analyze.

                default: 'amplitude'
                axis='phase'
                axis='imag'
                axis='scan_number'
                axis='flag'

                The phase of a complex number is in radians in the range [-pi; pi[.


        datacolumn -- Which data column to use for complex data.
                default: 'data'
                datacolumn='data'
                datacolumn='corrected'
                datacolumn='model'

        useflags -- Take MS flags into account?
                default: True
                useflag=False
                useflag=True
        If useflags=False, flagged values are included in the statistics.
        If useflags=True, any flagged values are not used in the statistics.

        spw -- Select data based on spectral window and channels
                default: '' (all); example: spw='1'
                spw='<2' #spectral windows less than 2
                spw='>1' #spectral windows greater than 1
                spw='0:0~10' # first 10 channels from spw 0
                spw='0:0~5;56~60' # multiple separated channel chunks.

        field -- Select data based on field id(s) or name(s)
                default: '' (all); example: field='1'
                field='0~2' # field ids inclusive from 0 to 2
                field='3C*' # all field names starting with 3C

        selectdata -- Other data selection parameters
                default: True
        antenna -- Select data based on baseline
                default: '' (all); example: antenna='5&6' baseline 5-6
                antenna='5&6;7&8' #baseline 5-6 and 7-8
                antenna='5' # all baselines with antenna 5
                antenna='5,6' # all baselines with antennas 5 and 6
        correlation -- Correlation types
                default: '' (all);
                example: correlation='RR LL'
        uvrange -- Select data within uvrange (default units meters)
                default: '' (all); example:
                uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lambda
                uvrange='>4klambda';uvranges greater than 4 kilo-lambda
                uvrange='0~1000km'; uvrange in kilometers
        timerange  -- Select data based on time range:
                default = '' (all); example,
                timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                Note: YYYY/MM/DD can be dropped as needed:
                timerange='09:14:0~09:54:0' # this time range
                timerange='09:44:00' # data within one integration of time
                timerange='>10:24:00' # data after this time
                timerange='09:44:00+00:13:00' #data 13 minutes after time
        scan -- Select data based on scan number
                default: '' (all); example: scan='>3'
        array -- Selection based on the antenna array
        observation -- Selection by observation ID(s).
	               default: '' (all); example: observation='1~3'


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'visstat'
        myf['taskname'] = 'visstat'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['axis'] = axis = myf['axis']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['useflags'] = useflags = myf['useflags']
            myparams['spw'] = spw = myf['spw']
            myparams['field'] = field = myf['field']
            myparams['selectdata'] = selectdata = myf['selectdata']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['correlation'] = correlation = myf['correlation']
            myparams['scan'] = scan = myf['scan']
            myparams['array'] = array = myf['array']
            myparams['observation'] = observation = myf['observation']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['axis'] = axis
        mytmp['datacolumn'] = datacolumn
        mytmp['useflags'] = useflags
        mytmp['spw'] = spw
        mytmp['field'] = field
        mytmp['selectdata'] = selectdata
        mytmp['antenna'] = antenna
        mytmp['uvrange'] = uvrange
        mytmp['timerange'] = timerange
        mytmp['correlation'] = correlation
        mytmp['scan'] = scan
        mytmp['array'] = array
        mytmp['observation'] = observation
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'visstat.xml')

        casalog.origin('visstat')
        if not trec.has_key('visstat') or not casac.utils().verify(mytmp, trec['visstat']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: visstat           #####')
          casalog.post('')
          result = visstat(vis, axis, datacolumn, useflags, spw, field, selectdata, antenna, uvrange, timerange, correlation, scan, array, observation)
          casalog.post('')
          casalog.post('##### End Task: visstat           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('visstat', 'visstat.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'visstat'
	     casalog.post('An error occurred running task %s: %s' % (tname,instance), 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('visstat', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['axis']  = 'amplitude'
        a['useflags']  = True
        a['spw']  = ''
        a['field']  = ''
        a['selectdata']  = True

        a['axis'] = {
                    0:odict([{'value':'amp'}, {'datacolumn':'data'}]), 
                    1:odict([{'value':'amplitude'}, {'datacolumn':'data'}]), 
                    2:odict([{'value':'phase'}, {'datacolumn':'data'}]), 
                    3:odict([{'value':'real'}, {'datacolumn':'data'}]), 
                    4:odict([{'value':'imag'}, {'datacolumn':'data'}]), 
                    5:odict([{'value':'imaginary'}, {'datacolumn':'data'}])}
        a['selectdata'] = {
                    0:odict([{'value':True}, {'antenna':''}, {'timerange':''}, {'correlation':''}, {'scan':''}, {'array':''}, {'observation':''}, {'uvrange':''}]), 
                    1:{'value':False}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='visstat', subkey=None):
        desc={'visstat': 'Displays statistical information from a Measurement Set, or from a Multi-MS',
               'vis': 'Name of Measurement Set or Multi-MS',
               'axis': 'Which values to use',
               'datacolumn': 'Which data column to use (data, corrected, model)',
               'useflags': 'Take flagging into account?',
               'spw': 'spectral-window/frequency/channel',
               'field': 'Field names or field index numbers: \'\'==>all, field=\'0~2,3C286\'',
               'selectdata': 'More data selection parameters (antenna, timerange etc)',
               'antenna': 'antenna/baselines: \'\'==>all, antenna = \'3,VA04\'',
               'uvrange': 'uv range: \'\'==>all; uvrange = \'0~100klambda\', default units=meters',
               'timerange': 'time range: \'\'==>all, timerange=\'09:14:0~09:54:0\'',
               'correlation': 'Select data based on correlation',
               'scan': 'scan numbers: \'\'==>all',
               'array': '(sub)array numbers: \'\'==>all',
               'observation': 'observation ID number(s): \'\' = all',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['axis']  = 'amplitude'
        a['datacolumn']  = 'data'
        a['useflags']  = True
        a['spw']  = ''
        a['field']  = ''
        a['selectdata']  = True
        a['antenna']  = ''
        a['uvrange']  = ''
        a['timerange']  = ''
        a['correlation']  = ''
        a['scan']  = ''
        a['array']  = ''
        a['observation']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
visstat_pg = visstat_pg_()
